@@ -20,6 +20,7 @@
 
 /**
  * SECTION:element-theoraparse
+ * @title: theoraparse
  * @see_also: theoradec, oggdemux, vorbisparse
  *
  * The theoraparse element will parse the header packets of the Theora
@@ -40,20 +41,19 @@
  * offsetting all buffers that it outputs by a specified amount, and updating
  * that offset from the value array whenever a keyframe is processed.
  *
- * <refsect2>
- * <title>Example pipelines</title>
+ * ## Example pipelines
  * |[
- * gst-launch -v filesrc location=video.ogg ! oggdemux ! theoraparse ! fakesink
- * ]| This pipeline shows that the streamheader is set in the caps, and that each
+ * gst-launch-1.0 -v filesrc location=video.ogg ! oggdemux ! theoraparse ! fakesink
+ * ]|
+ *  This pipeline shows that the streamheader is set in the caps, and that each
  * buffer has the timestamp, duration, offset, and offset_end set.
  * |[
- * gst-launch filesrc location=video.ogg ! oggdemux ! theoraparse \
+ * gst-launch-1.0 filesrc location=video.ogg ! oggdemux ! theoraparse \
  *            ! oggmux ! filesink location=video-remuxed.ogg
- * ]| This pipeline shows remuxing. video-remuxed.ogg might not be exactly the same
+ * ]|
+ *  This pipeline shows remuxing. video-remuxed.ogg might not be exactly the same
  * as video.ogg, but they should produce exactly the same decoded data.
- * </refsect2>
  *
- * Last reviewed on 2008-05-28 (0.10.20)
  */
 
 /* FIXME 0.11: suppress warnings for deprecated API such as GValueArray
@@ -126,8 +126,6 @@ gst_theora_parse_class_init (GstTheoraParseClass * klass)
    * GstTheoraParse:sychronization-points
    *
    * An array of (granuletime, buffertime) pairs
-   *
-   * Since: 0.10.10
    */
   g_object_class_install_property (gobject_class, PROP_SYNCHRONIZATION_POINTS,
       g_param_spec_value_array ("synchronization-points",
@@ -139,13 +137,13 @@ gst_theora_parse_class_init (GstTheoraParseClass * klass)
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 #endif
 
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&theora_parse_src_factory));
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&theora_parse_sink_factory));
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &theora_parse_src_factory);
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &theora_parse_sink_factory);
   gst_element_class_set_static_metadata (gstelement_class,
-      "Theora video parser", "Codec/Parser/Video",
-      "parse raw theora streams", "Andy Wingo <wingo@pobox.com>");
+      "Theora video parser", "Codec/Parser/Video", "parse raw theora streams",
+      "Andy Wingo <wingo@pobox.com>");
 
   gstelement_class->change_state = theora_parse_change_state;
 
@@ -326,7 +324,7 @@ theora_parse_set_streamheader (GstTheoraParse * parse)
     ret = th_decode_headerin (&parse->info, &parse->comment, &setup, &packet);
     gst_buffer_unmap (buf, &map);
     if (ret < 0) {
-      GST_WARNING_OBJECT (parse, "Failed to decode Theora header %d: %d\n",
+      GST_WARNING_OBJECT (parse, "Failed to decode Theora header %d: %d",
           i + 1, ret);
     }
   }
