@@ -34,9 +34,9 @@
  * Since: 1.2
  */
 
-#ifdef HAVE_MMAP
-#include <sys/mman.h>
-#include <unistd.h>
+#ifdef HAVE_LINUX_DMA_BUF_H
+#include <sys/ioctl.h>
+#include <linux/dma-buf.h>
 #endif
 
 GST_DEBUG_CATEGORY_STATIC (dmabuf_debug);
@@ -44,6 +44,58 @@ GST_DEBUG_CATEGORY_STATIC (dmabuf_debug);
 
 G_DEFINE_TYPE (GstDmaBufAllocator, gst_dmabuf_allocator, GST_TYPE_FD_ALLOCATOR);
 
+static gpointer
+gst_dmabuf_mem_map (GstMemory * gmem, GstMapInfo * info, gsize maxsize)
+{
+  GstAllocator *allocator = gmem->allocator;
+  gpointer ret;
+
+#ifdef HAVE_LINUX_DMA_BUF_H
+  struct dma_buf_sync sync = { DMA_BUF_SYNC_START };
+
+  if (info->flags & GST_MAP_READ)
+    sync.flags |= DMA_BUF_SYNC_READ;
+
+  if (info->flags & GST_MAP_WRITE)
+    sync.flags |= DMA_BUF_SYNC_WRITE;
+#endif
+
+  ret = allocator->mem_map (gmem, maxsize, info->flags);
+
+#ifdef HAVE_LINUX_DMA_BUF_H
+  if (ret) {
+    if (ioctl (gst_fd_memory_get_fd (gmem), DMA_BUF_IOCTL_SYNC, &sync) < 0)
+      GST_WARNING_OBJECT (allocator, "Failed to synchronize DMABuf: %s (%i)",
+          g_strerror (errno), errno);
+  }
+#endif
+
+  return ret;
+}
+
+static void
+gst_dmabuf_mem_unmap (GstMemory * gmem, GstMapInfo * info)
+{
+  GstAllocator *allocator = gmem->allocator;
+#ifdef HAVE_LINUX_DMA_BUF_H
+  struct dma_buf_sync sync = { DMA_BUF_SYNC_END };
+
+  if (info->flags & GST_MAP_READ)
+    sync.flags |= DMA_BUF_SYNC_READ;
+
+  if (info->flags & GST_MAP_WRITE)
+    sync.flags |= DMA_BUF_SYNC_WRITE;
+
+  if (ioctl (gst_fd_memory_get_fd (gmem), DMA_BUF_IOCTL_SYNC, &sync) < 0)
+    GST_WARNING_OBJECT (allocator, "Failed to synchronize DMABuf: %s (%i)",
+        g_strerror (errno), errno);
+#else
+  GST_WARNING_OBJECT (allocator, "Using DMABuf without synchronization.");
+#endif
+
+  allocator->mem_unmap (gmem);
+}
+
 static void
 gst_dmabuf_allocator_class_init (GstDmaBufAllocatorClass * klass)
 {
@@ -55,6 +107,8 @@ gst_dmabuf_allocator_init (GstDmaBufAllocator * allocator)
   GstAllocator *alloc = GST_ALLOCATOR_CAST (allocator);
 
   alloc->mem_type = GST_ALLOCATOR_DMABUF;
+  alloc->mem_map_full = gst_dmabuf_mem_map;
+  alloc->mem_unmap_full = gst_dmabuf_mem_unmap;
 }
 
 /**
@@ -91,7 +145,7 @@ gst_dmabuf_allocator_new (void)
  *
  * Returns: (transfer full): a GstMemory based on @allocator.
  * When the buffer will be released dmabuf allocator will close the @fd.
- * The memory is only mmapped on gst_buffer_mmap() request.
+ * The memory is only mmapped on gst_buffer_map() request.
  *
  * Since: 1.2
  */
@@ -103,6 +157,32 @@ gst_dmabuf_allocator_alloc (GstAllocator * allocator, gint fd, gsize size)
   return gst_fd_allocator_alloc (allocator, fd, size, GST_FD_MEMORY_FLAG_NONE);
 }
 
+/**
+ * gst_dmabuf_allocator_alloc_with_flags:
+ * @allocator: allocator to be used for this memory
+ * @fd: dmabuf file descriptor
+ * @size: memory size
+ * @flags: extra #GstFdMemoryFlags
+ *
+ * Return a %GstMemory that wraps a dmabuf file descriptor.
+ *
+ * Returns: (transfer full): a GstMemory based on @allocator.
+ *
+ * When the buffer will be released the allocator will close the @fd unless
+ * the %GST_FD_MEMORY_FLAG_DONT_CLOSE flag is specified.
+ * The memory is only mmapped on gst_buffer_mmap() request.
+ *
+ * Since: 1.16
+ */
+GstMemory *
+gst_dmabuf_allocator_alloc_with_flags (GstAllocator * allocator, gint fd,
+    gsize size, GstFdMemoryFlags flags)
+{
+  g_return_val_if_fail (GST_IS_DMABUF_ALLOCATOR (allocator), NULL);
+
+  return gst_fd_allocator_alloc (allocator, fd, size, flags);
+}
+
 /**
  * gst_dmabuf_memory_get_fd:
  * @mem: the memory to get the file descriptor
