@@ -40,7 +40,7 @@
  * to enable that behaviour, the GType of the sink pads must either be
  * a (subclass of) #GstAudioAggregatorConvertPad to use the default
  * #GstAudioConverter implementation, or a subclass of #GstAudioAggregatorPad
- * implementing #GstAudioAggregatorPad.convert_buffer.
+ * implementing #GstAudioAggregatorPadClass.convert_buffer.
  *
  * To allow for the output caps to change, the mechanism is the same as
  * above, with the GType of the source pad.
@@ -106,7 +106,7 @@ struct _GstAudioAggregatorPadPrivate
 /*****************************************
  * GstAudioAggregatorPad implementation  *
  *****************************************/
-G_DEFINE_TYPE (GstAudioAggregatorPad, gst_audio_aggregator_pad,
+G_DEFINE_TYPE_WITH_PRIVATE (GstAudioAggregatorPad, gst_audio_aggregator_pad,
     GST_TYPE_AGGREGATOR_PAD);
 
 enum
@@ -136,8 +136,6 @@ gst_audio_aggregator_pad_class_init (GstAudioAggregatorPadClass * klass)
   GObjectClass *gobject_class = (GObjectClass *) klass;
   GstAggregatorPadClass *aggpadclass = (GstAggregatorPadClass *) klass;
 
-  g_type_class_add_private (klass, sizeof (GstAudioAggregatorPadPrivate));
-
   gobject_class->finalize = gst_audio_aggregator_pad_finalize;
   aggpadclass->flush = GST_DEBUG_FUNCPTR (gst_audio_aggregator_pad_flush_pad);
 }
@@ -145,9 +143,7 @@ gst_audio_aggregator_pad_class_init (GstAudioAggregatorPadClass * klass)
 static void
 gst_audio_aggregator_pad_init (GstAudioAggregatorPad * pad)
 {
-  pad->priv =
-      G_TYPE_INSTANCE_GET_PRIVATE (pad, GST_TYPE_AUDIO_AGGREGATOR_PAD,
-      GstAudioAggregatorPadPrivate);
+  pad->priv = gst_audio_aggregator_pad_get_instance_private (pad);
 
   gst_audio_info_init (&pad->info);
 
@@ -187,8 +183,8 @@ struct _GstAudioAggregatorConvertPadPrivate
 };
 
 
-G_DEFINE_TYPE (GstAudioAggregatorConvertPad, gst_audio_aggregator_convert_pad,
-    GST_TYPE_AUDIO_AGGREGATOR_PAD);
+G_DEFINE_TYPE_WITH_PRIVATE (GstAudioAggregatorConvertPad,
+    gst_audio_aggregator_convert_pad, GST_TYPE_AUDIO_AGGREGATOR_PAD);
 
 static void
 gst_audio_aggregator_convert_pad_update_converter (GstAudioAggregatorConvertPad
@@ -337,8 +333,6 @@ gst_audio_aggregator_convert_pad_class_init (GstAudioAggregatorConvertPadClass *
   GObjectClass *gobject_class = (GObjectClass *) klass;
   GstAudioAggregatorPadClass *aaggpad_class =
       (GstAudioAggregatorPadClass *) klass;
-  g_type_class_add_private (klass,
-      sizeof (GstAudioAggregatorConvertPadPrivate));
 
   gobject_class->set_property = gst_audio_aggregator_convert_pad_set_property;
   gobject_class->get_property = gst_audio_aggregator_convert_pad_get_property;
@@ -361,9 +355,7 @@ gst_audio_aggregator_convert_pad_class_init (GstAudioAggregatorConvertPadClass *
 static void
 gst_audio_aggregator_convert_pad_init (GstAudioAggregatorConvertPad * pad)
 {
-  pad->priv =
-      G_TYPE_INSTANCE_GET_PRIVATE (pad, GST_TYPE_AUDIO_AGGREGATOR_CONVERT_PAD,
-      GstAudioAggregatorConvertPadPrivate);
+  pad->priv = gst_audio_aggregator_convert_pad_get_instance_private (pad);
 }
 
 /**************************************
@@ -441,30 +433,9 @@ enum
   PROP_DISCONT_WAIT,
 };
 
-G_DEFINE_ABSTRACT_TYPE (GstAudioAggregator, gst_audio_aggregator,
+G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE (GstAudioAggregator, gst_audio_aggregator,
     GST_TYPE_AGGREGATOR);
 
-static GstClockTime
-gst_audio_aggregator_get_next_time (GstAggregator * agg)
-{
-  GstClockTime next_time;
-  GstSegment *segment = &GST_AGGREGATOR_PAD (agg->srcpad)->segment;
-
-  GST_OBJECT_LOCK (agg);
-  if (segment->position == -1 || segment->position < segment->start)
-    next_time = segment->start;
-  else
-    next_time = segment->position;
-
-  if (segment->stop != -1 && next_time > segment->stop)
-    next_time = segment->stop;
-
-  next_time = gst_segment_to_running_time (segment, GST_FORMAT_TIME, next_time);
-  GST_OBJECT_UNLOCK (agg);
-
-  return next_time;
-}
-
 static GstBuffer *
 gst_audio_aggregator_convert_buffer (GstAudioAggregator * aagg, GstPad * pad,
     GstAudioInfo * in_info, GstAudioInfo * out_info, GstBuffer * buffer)
@@ -483,8 +454,6 @@ gst_audio_aggregator_class_init (GstAudioAggregatorClass * klass)
   GObjectClass *gobject_class = (GObjectClass *) klass;
   GstAggregatorClass *gstaggregator_class = (GstAggregatorClass *) klass;
 
-  g_type_class_add_private (klass, sizeof (GstAudioAggregatorPrivate));
-
   gobject_class->set_property = gst_audio_aggregator_set_property;
   gobject_class->get_property = gst_audio_aggregator_get_property;
   gobject_class->dispose = gst_audio_aggregator_dispose;
@@ -502,7 +471,7 @@ gst_audio_aggregator_class_init (GstAudioAggregatorClass * klass)
   gstaggregator_class->aggregate =
       GST_DEBUG_FUNCPTR (gst_audio_aggregator_aggregate);
   gstaggregator_class->clip = GST_DEBUG_FUNCPTR (gst_audio_aggregator_do_clip);
-  gstaggregator_class->get_next_time = gst_audio_aggregator_get_next_time;
+  gstaggregator_class->get_next_time = gst_aggregator_simple_get_next_time;
   gstaggregator_class->update_src_caps =
       GST_DEBUG_FUNCPTR (gst_audio_aggregator_update_src_caps);
   gstaggregator_class->fixate_src_caps = gst_audio_aggregator_fixate_src_caps;
@@ -537,9 +506,7 @@ gst_audio_aggregator_class_init (GstAudioAggregatorClass * klass)
 static void
 gst_audio_aggregator_init (GstAudioAggregator * aagg)
 {
-  aagg->priv =
-      G_TYPE_INSTANCE_GET_PRIVATE (aagg, GST_TYPE_AUDIO_AGGREGATOR,
-      GstAudioAggregatorPrivate);
+  aagg->priv = gst_audio_aggregator_get_instance_private (aagg);
 
   g_mutex_init (&aagg->priv->mutex);
 
@@ -644,46 +611,134 @@ gst_audio_aggregator_sink_getcaps (GstPad * pad, GstAggregator * agg,
   GstCaps *sink_template_caps = gst_pad_get_pad_template_caps (pad);
   GstCaps *downstream_caps = gst_pad_get_allowed_caps (agg->srcpad);
   GstCaps *sink_caps;
-  GstStructure *s, *s2;
-  gint downstream_rate;
-
-  sink_template_caps = gst_caps_make_writable (sink_template_caps);
-  s = gst_caps_get_structure (sink_template_caps, 0);
-
-  /* We will then use the rate in the first structure as the expected
-   * rate, we want to make sure only the compatible structures remain
-   * in downstream_caps
-   */
-  if (downstream_caps && filter) {
-    GstCaps *tmp = gst_caps_intersect_full (downstream_caps, filter,
-        GST_CAPS_INTERSECT_FIRST);
-
-    gst_caps_unref (downstream_caps);
-    downstream_caps = tmp;
-  }
-
-  if (downstream_caps && !gst_caps_is_empty (downstream_caps))
-    s2 = gst_caps_get_structure (downstream_caps, 0);
-  else
-    s2 = NULL;
-
-  if (s2 && gst_structure_get_int (s2, "rate", &downstream_rate)) {
-    gst_structure_fixate_field_nearest_int (s, "rate", downstream_rate);
-  } else if (first_configured_pad) {
-    gst_structure_fixate_field_nearest_int (s, "rate",
-        first_configured_pad->info.rate);
-  }
-
-  if (first_configured_pad)
-    gst_object_unref (first_configured_pad);
-
-  sink_caps = filter ? gst_caps_intersect (sink_template_caps,
-      filter) : gst_caps_ref (sink_template_caps);
 
   GST_INFO_OBJECT (pad, "Getting caps with filter %" GST_PTR_FORMAT, filter);
   GST_DEBUG_OBJECT (pad, "sink template caps : %" GST_PTR_FORMAT,
       sink_template_caps);
   GST_DEBUG_OBJECT (pad, "downstream caps %" GST_PTR_FORMAT, downstream_caps);
+
+  /* If we already have a configured pad, assume that we can only configure
+   * to the very same format filtered with the template caps and continue
+   * with the result of that as the template caps */
+
+  if (first_configured_pad) {
+    GstCaps *first_configured_caps =
+        gst_audio_info_to_caps (&first_configured_pad->info);
+    GstCaps *tmp;
+
+    tmp =
+        gst_caps_intersect_full (sink_template_caps, first_configured_caps,
+        GST_CAPS_INTERSECT_FIRST);
+    gst_caps_unref (first_configured_caps);
+    gst_caps_unref (sink_template_caps);
+    sink_template_caps = tmp;
+
+    gst_object_unref (first_configured_pad);
+  }
+
+  /* If we have downstream caps, filter them against our template caps or
+   * the filtered first configured pad caps from above */
+  if (downstream_caps) {
+    sink_caps =
+        gst_caps_intersect_full (sink_template_caps, downstream_caps,
+        GST_CAPS_INTERSECT_FIRST);
+  } else {
+    sink_caps = gst_caps_ref (sink_template_caps);
+  }
+
+  if (filter) {
+    GstCaps *tmp = gst_caps_intersect_full (sink_caps, filter,
+        GST_CAPS_INTERSECT_FIRST);
+
+    gst_caps_unref (sink_caps);
+    sink_caps = tmp;
+  }
+
+  gst_caps_unref (sink_template_caps);
+
+  if (downstream_caps)
+    gst_caps_unref (downstream_caps);
+
+  GST_INFO_OBJECT (pad, "returned sink caps : %" GST_PTR_FORMAT, sink_caps);
+
+  return sink_caps;
+}
+
+static GstCaps *
+gst_audio_aggregator_convert_sink_getcaps (GstPad * pad, GstAggregator * agg,
+    GstCaps * filter)
+{
+  GstAudioAggregatorPad *first_configured_pad =
+      gst_audio_aggregator_get_first_configured_pad (agg);
+  GstCaps *sink_template_caps = gst_pad_get_pad_template_caps (pad);
+  GstCaps *downstream_caps = gst_pad_get_allowed_caps (agg->srcpad);
+  GstCaps *sink_caps;
+
+  GST_INFO_OBJECT (pad, "Getting caps with filter %" GST_PTR_FORMAT, filter);
+  GST_DEBUG_OBJECT (pad, "sink template caps : %" GST_PTR_FORMAT,
+      sink_template_caps);
+  GST_DEBUG_OBJECT (pad, "downstream caps %" GST_PTR_FORMAT, downstream_caps);
+
+  /* We can convert between all formats except for the sample rate, which has
+   * to match.  */
+
+  /* If we have a first configured pad, we can only convert everything except
+   * for the sample rate, so modify our template caps to have exactly that
+   * sample rate in all structures */
+  if (first_configured_pad) {
+    GST_INFO_OBJECT (pad, "first configured pad has sample rate %d",
+        first_configured_pad->info.rate);
+    sink_template_caps = gst_caps_make_writable (sink_template_caps);
+    gst_caps_set_simple (sink_template_caps, "rate", G_TYPE_INT,
+        first_configured_pad->info.rate, NULL);
+    gst_object_unref (first_configured_pad);
+  }
+
+  /* Now if we have downstream caps, filter against the template caps from
+   * above, i.e. with potentially fixated sample rate field already. This
+   * filters out any structures with unsupported rates.
+   *
+   * Afterwards we create new caps that only take over the rate fields of the
+   * remaining downstream caps, and filter that against the plain template
+   * caps to get the resulting allowed caps with conversion for everything but
+   * the rate */
+  if (downstream_caps) {
+    GstCaps *tmp;
+    guint i, n;
+
+    tmp =
+        gst_caps_intersect_full (sink_template_caps, downstream_caps,
+        GST_CAPS_INTERSECT_FIRST);
+
+    n = gst_caps_get_size (tmp);
+    sink_caps = gst_caps_new_empty ();
+    for (i = 0; i < n; i++) {
+      GstStructure *s = gst_caps_get_structure (tmp, i);
+      GstStructure *new_s =
+          gst_structure_new_empty (gst_structure_get_name (s));
+      gst_structure_set_value (new_s, "rate", gst_structure_get_value (s,
+              "rate"));
+      sink_caps = gst_caps_merge_structure (sink_caps, new_s);
+    }
+    gst_caps_unref (tmp);
+    tmp = sink_caps;
+
+    sink_caps =
+        gst_caps_intersect_full (sink_template_caps, tmp,
+        GST_CAPS_INTERSECT_FIRST);
+    gst_caps_unref (tmp);
+  } else {
+    sink_caps = gst_caps_ref (sink_template_caps);
+  }
+
+  /* And finally filter anything that remains against the filter caps */
+  if (filter) {
+    GstCaps *tmp =
+        gst_caps_intersect_full (filter, sink_caps, GST_CAPS_INTERSECT_FIRST);
+    gst_caps_unref (sink_caps);
+    sink_caps = tmp;
+  }
+
   GST_INFO_OBJECT (pad, "returned sink caps : %" GST_PTR_FORMAT, sink_caps);
 
   gst_caps_unref (sink_template_caps);
@@ -844,7 +899,7 @@ gst_audio_aggregator_fixate_src_caps (GstAggregator * agg, GstCaps * caps)
 /* Must be called with OBJECT_LOCK taken */
 static void
 gst_audio_aggregator_update_converters (GstAudioAggregator * aagg,
-    GstAudioInfo * new_info)
+    GstAudioInfo * new_info, GstAudioInfo * old_info)
 {
   GList *l;
 
@@ -861,7 +916,7 @@ gst_audio_aggregator_update_converters (GstAudioAggregator * aagg,
     if (aaggpad->priv->buffer) {
       GstBuffer *new_converted_buffer =
           gst_audio_aggregator_convert_buffer (aagg, GST_PAD (aaggpad),
-          &aaggpad->info, new_info, aaggpad->priv->input_buffer);
+          old_info, new_info, aaggpad->priv->input_buffer);
       gst_buffer_replace (&aaggpad->priv->buffer, new_converted_buffer);
       gst_buffer_unref (new_converted_buffer);
     }
@@ -886,31 +941,31 @@ gst_audio_aggregator_negotiated_src_caps (GstAggregator * agg, GstCaps * caps)
   GST_AUDIO_AGGREGATOR_LOCK (aagg);
   GST_OBJECT_LOCK (aagg);
 
-  if (GST_AUDIO_AGGREGATOR_PAD_GET_CLASS (agg->srcpad)->convert_buffer) {
-    gst_audio_aggregator_update_converters (aagg, &info);
-
-    if (aagg->priv->current_buffer
-        && !gst_audio_info_is_equal (&srcpad->info, &info)) {
-      GstBuffer *converted;
-      GstAudioAggregatorPadClass *klass =
-          GST_AUDIO_AGGREGATOR_PAD_GET_CLASS (agg->srcpad);
-
-      if (klass->update_conversion_info)
-        klass->update_conversion_info (GST_AUDIO_AGGREGATOR_PAD (agg->srcpad));
-
-      converted =
-          gst_audio_aggregator_convert_buffer (aagg, agg->srcpad, &srcpad->info,
-          &info, aagg->priv->current_buffer);
-      gst_buffer_unref (aagg->priv->current_buffer);
-      aagg->priv->current_buffer = converted;
-    }
-  }
-
   if (!gst_audio_info_is_equal (&info, &srcpad->info)) {
+    GstAudioInfo old_info = srcpad->info;
+    GstAudioAggregatorPadClass *srcpad_klass =
+        GST_AUDIO_AGGREGATOR_PAD_GET_CLASS (agg->srcpad);
+
     GST_INFO_OBJECT (aagg, "setting caps to %" GST_PTR_FORMAT, caps);
     gst_caps_replace (&aagg->current_caps, caps);
 
     memcpy (&srcpad->info, &info, sizeof (info));
+
+    gst_audio_aggregator_update_converters (aagg, &info, &old_info);
+
+    if (srcpad_klass->update_conversion_info)
+      srcpad_klass->
+          update_conversion_info (GST_AUDIO_AGGREGATOR_PAD (agg->srcpad));
+
+    if (aagg->priv->current_buffer) {
+      GstBuffer *converted;
+
+      converted =
+          gst_audio_aggregator_convert_buffer (aagg, agg->srcpad, &old_info,
+          &info, aagg->priv->current_buffer);
+      gst_buffer_unref (aagg->priv->current_buffer);
+      aagg->priv->current_buffer = converted;
+    }
   }
 
   GST_OBJECT_UNLOCK (aagg);
@@ -1059,6 +1114,12 @@ gst_audio_aggregator_sink_event (GstAggregator * agg,
       break;
   }
 
+  if (!res) {
+    if (event)
+      gst_event_unref (event);
+    return res;
+  }
+
   if (event != NULL)
     return
         GST_AGGREGATOR_CLASS (gst_audio_aggregator_parent_class)->sink_event
@@ -1079,7 +1140,14 @@ gst_audio_aggregator_sink_query (GstAggregator * agg, GstAggregatorPad * aggpad,
       GstCaps *filter, *caps;
 
       gst_query_parse_caps (query, &filter);
-      caps = gst_audio_aggregator_sink_getcaps (GST_PAD (aggpad), agg, filter);
+      if (GST_IS_AUDIO_AGGREGATOR_CONVERT_PAD (aggpad)) {
+        caps =
+            gst_audio_aggregator_convert_sink_getcaps (GST_PAD (aggpad), agg,
+            filter);
+      } else {
+        caps =
+            gst_audio_aggregator_sink_getcaps (GST_PAD (aggpad), agg, filter);
+      }
       gst_query_set_caps_result (query, caps);
       gst_caps_unref (caps);
       res = TRUE;
@@ -1321,10 +1389,13 @@ gst_audio_aggregator_do_clip (GstAggregator * agg,
   GstAudioAggregatorPad *pad = GST_AUDIO_AGGREGATOR_PAD (bpad);
   gint rate, bpf;
 
-  rate = GST_AUDIO_INFO_RATE (&pad->info);
-  bpf = GST_AUDIO_INFO_BPF (&pad->info);
+  /* Guard against invalid audio info, we just don't clip here then */
+  if (!GST_AUDIO_INFO_IS_VALID (&pad->info))
+    return buffer;
 
   GST_OBJECT_LOCK (bpad);
+  rate = GST_AUDIO_INFO_RATE (&pad->info);
+  bpf = GST_AUDIO_INFO_BPF (&pad->info);
   buffer = gst_audio_buffer_clip (buffer, &bpad->segment, rate, bpf);
   GST_OBJECT_UNLOCK (bpad);
 
@@ -1820,6 +1891,10 @@ gst_audio_aggregator_aggregate (GstAggregator * agg, gboolean timeout)
       }
       GST_OBJECT_UNLOCK (pad);
       continue;
+    } else if (!GST_AUDIO_INFO_IS_VALID (&pad->info)) {
+      GST_OBJECT_UNLOCK (pad);
+      GST_OBJECT_UNLOCK (agg);
+      goto not_negotiated;
     }
 
     /* New buffer? */
