@@ -31,16 +31,12 @@ struct _GstDiscovererStreamInfo {
   GstToc                *toc;
   gchar                 *stream_id;
   GstStructure          *misc;
-
-  gpointer _gst_reserved[GST_PADDING];
 };
 
 struct _GstDiscovererContainerInfo {
   GstDiscovererStreamInfo parent;
 
   GList               *streams;
-
-  gpointer _gst_reserved[GST_PADDING];
 };
 
 struct _GstDiscovererAudioInfo {
@@ -55,8 +51,6 @@ struct _GstDiscovererAudioInfo {
   guint max_bitrate;
 
   gchar *language;
-
-  gpointer _gst_reserved[GST_PADDING];
 };
 
 struct _GstDiscovererVideoInfo {
@@ -75,16 +69,12 @@ struct _GstDiscovererVideoInfo {
   guint max_bitrate;
 
   gboolean is_image;
-
-  gpointer _gst_reserved[GST_PADDING];
 };
 
 struct _GstDiscovererSubtitleInfo {
   GstDiscovererStreamInfo parent;
 
   gchar *language;
-
-  gpointer _gst_reserved[GST_PADDING];
 };
 
 struct _GstDiscovererInfo {
@@ -106,7 +96,8 @@ struct _GstDiscovererInfo {
   gboolean seekable;
   GPtrArray *missing_elements_details;
 
-  gpointer _gst_reserved[GST_PADDING];
+  gchar *cachefile;
+  gpointer from_cache;
 };
 
 /* missing-plugins.c */
