@@ -29,6 +29,9 @@
  * 'application/x-rtp' #GstCaps.
  *
  */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include "gstrtpbuffer.h"
 
