@@ -18,11 +18,21 @@
  * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include "gstvideoaffinetransformationmeta.h"
 
 #include <string.h>
 
+/**
+ * SECTION:gstvideoaffinetransformationmeta
+ * @title: GstVideoAffineTransformationMeta
+ * @short_description: GstMeta for video affine transformation
+ *
+ */
+
 GType
 gst_video_affine_transformation_meta_api_get_type (void)
 {
@@ -128,7 +138,7 @@ gst_buffer_add_video_affine_transformation_meta (GstBuffer * buffer)
 /**
  * gst_video_affine_transformation_meta_apply_matrix:
  * @meta: a #GstVideoAffineTransformationMeta
- * @matrix: a 4x4 transformation matrix to be applied
+ * @matrix: (array fixed-size=16): a 4x4 transformation matrix to be applied
  *
  * Apply a transformation using the given 4x4 transformation matrix.
  * Performs the multiplication, meta->matrix X matrix.
