@@ -18,22 +18,76 @@
  */
 /**
  * SECTION:element-audiotestsrc
+ * @title: audiotestsrc
  *
  * AudioTestSrc can be used to generate basic audio signals. It support several
- * different waveforms and allows to set the base frequency and volume.
+ * different waveforms and allows to set the base frequency and volume. Some
+ * waveforms might use additional properties.
  *
- * <refsect2>
- * <title>Example launch line</title>
+ * Waveform specific notes:
+ *
+ * <orderedlist>
+ *   <listitem>
+ *     <itemizedlist><title>Gaussian white noise</title>
+ *     
+ *     This waveform produces white (zero mean) Gaussian noise.
+ *     Volume sets the standard deviation of the noise in units of the range
+ *     of values of the sample type, e.g. volume=0.1 produces noise with a
+ *     standard deviation of 0.1*32767=3277 with 16-bit integer samples,
+ *     or 0.1*1.0=0.1 with floating-point samples.
+ *     
+ *     </itemizedlist>
+ *   </listitem>
+ *   <listitem>
+ *     <itemizedlist><title>Ticks</title>
+ *     
+ *     This waveform is special in that it does not produce one continuous
+ *     signal. Instead, it produces finite-length sine wave pulses (the "ticks").
+ *     It is useful for detecting time shifts between audio signal, for example
+ *     between RTSP audio clients that shall play synchronized. It is also useful
+ *     for generating a signal that feeds the trigger of an oscilloscope.
+ *
+ *     To further help with oscilloscope triggering and time offset detection,
+ *     the waveform can apply a different volume to every Nth tick (this is then
+ *     called the "marker tick"). For instance, one could generate a tick every
+ *     100ms, and make every 20th tick a marker tick (meaning that every 2 seconds
+ *     there is a marker tick). This is useful for detecting large time offsets
+ *     while still frequently triggering an oscilloscope.
+ *
+ *     Also, a "ramp" can be applied to the begin & end of ticks. The sudden
+ *     start of the sine tick is a discontinuity, even if the sine wave starts
+ *     at 0. The* resulting artifacts can often make it more difficult to use the
+ *     ticks for an oscilloscope's trigger. To that end, an initial "ramp" can
+ *     be applied. The first few samples are modulated by a cubic function to
+ *     reduce the impact of the discontinuity, resulting in smaller artifacts.
+ *     The number of samples equals floor(samplerate / sine-wave-frequency).
+ *     Example: with a sample rate of 48 kHz and a sine wave frequency of 10 kHz,
+ *     the first 4 samples are modulated by the cubic function.
+ *     </itemizedlist>
+ *   </listitem>
+ * </orderedlist>
+ *
+ * ## Example launch line
  * |[
  * gst-launch-1.0 audiotestsrc ! audioconvert ! autoaudiosink
- * ]| This pipeline produces a sine with default frequency, 440 Hz, and the
+ * ]|
+ *  This pipeline produces a sine with default frequency, 440 Hz, and the
  * default volume, 0.8 (relative to a maximum 1.0).
  * |[
- * gst-launch-1.0 audiotestsrc wave=2 freq=200 ! tee name=t ! queue ! audioconvert ! autoaudiosink t. ! queue ! audioconvert ! libvisual_lv_scope ! videoconvert ! autovideosink
- * ]| In this example a saw wave is generated. The wave is shown using a
+ * gst-launch-1.0 audiotestsrc wave=2 freq=200 ! tee name=t ! queue ! audioconvert ! \
+ *     autoaudiosink t. ! queue ! audioconvert ! libvisual_lv_scope ! videoconvert ! autovideosink
+ * ]|
+ *  In this example a saw wave is generated. The wave is shown using a
  * scope visualizer from libvisual, allowing you to visually verify that
  * the saw wave is correct.
- * </refsect2>
+ *
+ * |[
+ * gst-launch-1.0 audiotestsrc wave=ticks apply-tick-ramp=true tick-interval=100000000 \
+ *     freq=10000 volume=0.4 marker-tick-period=10 sine-periods-per-tick=20 ! autoaudiosink
+ * ]| This pipeline produces a series of 10 kHz sine wave ticks. Each tick is
+ * 20 sine wave periods long, ticks occur every 100 ms and have a volume of
+ * 0.4. Every 10th tick is a marker tick and has the default marker tick volume
+ * of 1.0. The beginning and end of the ticks are modulated with the ramp.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -52,14 +106,19 @@
 GST_DEBUG_CATEGORY_STATIC (audio_test_src_debug);
 #define GST_CAT_DEFAULT audio_test_src_debug
 
-#define DEFAULT_SAMPLES_PER_BUFFER   1024
-#define DEFAULT_WAVE                 GST_AUDIO_TEST_SRC_WAVE_SINE
-#define DEFAULT_FREQ                 440.0
-#define DEFAULT_VOLUME               0.8
-#define DEFAULT_IS_LIVE              FALSE
-#define DEFAULT_TIMESTAMP_OFFSET     G_GINT64_CONSTANT (0)
-#define DEFAULT_CAN_ACTIVATE_PUSH    TRUE
-#define DEFAULT_CAN_ACTIVATE_PULL    FALSE
+#define DEFAULT_SAMPLES_PER_BUFFER      1024
+#define DEFAULT_WAVE                    GST_AUDIO_TEST_SRC_WAVE_SINE
+#define DEFAULT_FREQ                    440.0
+#define DEFAULT_VOLUME                  0.8
+#define DEFAULT_IS_LIVE                 FALSE
+#define DEFAULT_TIMESTAMP_OFFSET        G_GINT64_CONSTANT (0)
+#define DEFAULT_SINE_PERIODS_PER_TICK   10
+#define DEFAULT_TIME_BETWEEN_TICKS      GST_SECOND
+#define DEFAULT_MARKER_TICK_PERIOD    0
+#define DEFAULT_MARKER_TICK_VOLUME      1.0
+#define DEFAULT_APPLY_TICK_RAMP         FALSE
+#define DEFAULT_CAN_ACTIVATE_PUSH       TRUE
+#define DEFAULT_CAN_ACTIVATE_PULL       FALSE
 
 enum
 {
@@ -70,6 +129,11 @@ enum
   PROP_VOLUME,
   PROP_IS_LIVE,
   PROP_TIMESTAMP_OFFSET,
+  PROP_SINE_PERIODS_PER_TICK,
+  PROP_TICK_INTERVAL,
+  PROP_MARKER_TICK_PERIOD,
+  PROP_MARKER_TICK_VOLUME,
+  PROP_APPLY_TICK_RAMP,
   PROP_CAN_ACTIVATE_PUSH,
   PROP_CAN_ACTIVATE_PULL
 };
@@ -91,7 +155,7 @@ GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("audio/x-raw, "
         "format = (string) " FORMAT_STR ", "
-        "layout = (string) interleaved, "
+        "layout = (string) { interleaved, non-interleaved }, "
         "rate = " GST_AUDIO_RATE_RANGE ", "
         "channels = " GST_AUDIO_CHANNELS_RANGE)
     );
@@ -198,6 +262,32 @@ gst_audio_test_src_class_init (GstAudioTestSrcClass * klass)
           "An offset added to timestamps set on buffers (in ns)", G_MININT64,
           G_MAXINT64, DEFAULT_TIMESTAMP_OFFSET,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+  g_object_class_install_property (gobject_class, PROP_SINE_PERIODS_PER_TICK,
+      g_param_spec_uint ("sine-periods-per-tick", "Sine periods per tick",
+          "Number of sine wave periods in one tick. Only used if wave = ticks.",
+          1, G_MAXUINT, DEFAULT_SINE_PERIODS_PER_TICK,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+  g_object_class_install_property (gobject_class, PROP_TICK_INTERVAL,
+      g_param_spec_uint64 ("tick-interval", "Time between ticks",
+          "Distance between start of current and start of next tick, in nanoseconds.",
+          1, G_MAXUINT64, DEFAULT_TIME_BETWEEN_TICKS,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+  g_object_class_install_property (gobject_class, PROP_MARKER_TICK_PERIOD,
+      g_param_spec_uint ("marker-tick-period", "Marker tick period",
+          "Make every Nth tick a marker tick (= a tick with different volume). "
+          "Only used if wave = ticks. 0 = no marker ticks.",
+          0, G_MAXUINT, DEFAULT_MARKER_TICK_PERIOD,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+  g_object_class_install_property (gobject_class, PROP_MARKER_TICK_VOLUME,
+      g_param_spec_double ("marker-tick-volume", "Marker tick volume",
+          "Volume of marker ticks. Only used if wave = ticks and"
+          "marker-tick-period is set to a nonzero value.",
+          0.0, 1.0, DEFAULT_MARKER_TICK_VOLUME,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+  g_object_class_install_property (gobject_class, PROP_APPLY_TICK_RAMP,
+      g_param_spec_boolean ("apply-tick-ramp", "Apply tick ramp",
+          "Apply ramp to tick samples", DEFAULT_APPLY_TICK_RAMP,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_CAN_ACTIVATE_PUSH,
       g_param_spec_boolean ("can-activate-push", "Can activate push",
           "Can activate in push mode", DEFAULT_CAN_ACTIVATE_PUSH,
@@ -207,10 +297,10 @@ gst_audio_test_src_class_init (GstAudioTestSrcClass * klass)
           "Can activate in pull mode", DEFAULT_CAN_ACTIVATE_PULL,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&gst_audio_test_src_src_template));
-  gst_element_class_set_static_metadata (gstelement_class,
-      "Audio test source", "Source/Audio",
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &gst_audio_test_src_src_template);
+  gst_element_class_set_static_metadata (gstelement_class, "Audio test source",
+      "Source/Audio",
       "Creates audio test signals of given frequency and volume",
       "Stefan Kost <ensonic@users.sf.net>");
 
@@ -242,6 +332,12 @@ gst_audio_test_src_init (GstAudioTestSrc * src)
   src->timestamp_offset = DEFAULT_TIMESTAMP_OFFSET;
   src->can_activate_pull = DEFAULT_CAN_ACTIVATE_PULL;
 
+  src->sine_periods_per_tick = DEFAULT_SINE_PERIODS_PER_TICK;
+  src->tick_interval = DEFAULT_TIME_BETWEEN_TICKS;
+  src->marker_tick_period = DEFAULT_MARKER_TICK_PERIOD;
+  src->marker_tick_volume = DEFAULT_MARKER_TICK_VOLUME;
+  src->apply_tick_ramp = DEFAULT_APPLY_TICK_RAMP;
+
   src->gen = NULL;
 
   src->wave = DEFAULT_WAVE;
@@ -281,6 +377,8 @@ gst_audio_test_src_fixate (GstBaseSrc * bsrc, GstCaps * caps)
 
   gst_structure_fixate_field_string (structure, "format", DEFAULT_FORMAT_STR);
 
+  gst_structure_fixate_field_string (structure, "layout", "interleaved");
+
   /* fixate to mono unless downstream requires stereo, for backwards compat */
   gst_structure_fixate_field_nearest_int (structure, "channels", 1);
 
@@ -391,22 +489,32 @@ error:
 static void \
 gst_audio_test_src_create_sine_##type (GstAudioTestSrc * src, g##type * samples) \
 { \
-  gint i, c, channels; \
+  gint i, c, channels, channel_step, sample_step; \
   gdouble step, amp; \
+  g##type *ptr; \
   \
   channels = GST_AUDIO_INFO_CHANNELS (&src->info); \
+  if (GST_AUDIO_INFO_LAYOUT (&src->info) == GST_AUDIO_LAYOUT_INTERLEAVED) { \
+    channel_step = 1; \
+    sample_step = channels; \
+  } else { \
+    channel_step = src->generate_samples_per_buffer; \
+    sample_step = 1; \
+  } \
   step = M_PI_M2 * src->freq / GST_AUDIO_INFO_RATE (&src->info); \
   amp = src->volume * scale; \
   \
-  i = 0; \
-  while (i < (src->generate_samples_per_buffer * channels)) { \
+  for (i = 0; i < src->generate_samples_per_buffer; i++) { \
     src->accumulator += step; \
     if (src->accumulator >= M_PI_M2) \
       src->accumulator -= M_PI_M2; \
     \
+    ptr = samples; \
     for (c = 0; c < channels; ++c) { \
-      samples[i++] = (g##type) (sin (src->accumulator) * amp); \
+      *ptr = (g##type) (sin (src->accumulator) * amp); \
+      ptr += channel_step; \
     } \
+    samples += sample_step; \
   } \
 }
 
@@ -426,22 +534,32 @@ static const ProcessFunc sine_funcs[] = {
 static void \
 gst_audio_test_src_create_square_##type (GstAudioTestSrc * src, g##type * samples) \
 { \
-  gint i, c, channels; \
+  gint i, c, channels, channel_step, sample_step; \
   gdouble step, amp; \
+  g##type *ptr; \
   \
   channels = GST_AUDIO_INFO_CHANNELS (&src->info); \
+  if (GST_AUDIO_INFO_LAYOUT (&src->info) == GST_AUDIO_LAYOUT_INTERLEAVED) { \
+    channel_step = 1; \
+    sample_step = channels; \
+  } else { \
+    channel_step = src->generate_samples_per_buffer; \
+    sample_step = 1; \
+  } \
   step = M_PI_M2 * src->freq / GST_AUDIO_INFO_RATE (&src->info); \
   amp = src->volume * scale; \
   \
-  i = 0; \
-  while (i < (src->generate_samples_per_buffer * channels)) { \
+  for (i = 0; i < src->generate_samples_per_buffer; i++) { \
     src->accumulator += step; \
     if (src->accumulator >= M_PI_M2) \
       src->accumulator -= M_PI_M2; \
     \
+    ptr = samples; \
     for (c = 0; c < channels; ++c) { \
-      samples[i++] = (g##type) ((src->accumulator < G_PI) ? amp : -amp); \
+      *ptr = (g##type) ((src->accumulator < G_PI) ? amp : -amp); \
+      ptr += channel_step; \
     } \
+    samples += sample_step; \
   } \
 }
 
@@ -461,26 +579,39 @@ static const ProcessFunc square_funcs[] = {
 static void \
 gst_audio_test_src_create_saw_##type (GstAudioTestSrc * src, g##type * samples) \
 { \
-  gint i, c, channels; \
+  gint i, c, channels, channel_step, sample_step; \
   gdouble step, amp; \
+  g##type *ptr; \
   \
   channels = GST_AUDIO_INFO_CHANNELS (&src->info); \
+  if (GST_AUDIO_INFO_LAYOUT (&src->info) == GST_AUDIO_LAYOUT_INTERLEAVED) { \
+    channel_step = 1; \
+    sample_step = channels; \
+  } else { \
+    channel_step = src->generate_samples_per_buffer; \
+    sample_step = 1; \
+  } \
   step = M_PI_M2 * src->freq / GST_AUDIO_INFO_RATE (&src->info); \
   amp = (src->volume * scale) / G_PI; \
   \
-  i = 0; \
-  while (i < (src->generate_samples_per_buffer * channels)) { \
+  for (i = 0; i < src->generate_samples_per_buffer; i++) { \
     src->accumulator += step; \
     if (src->accumulator >= M_PI_M2) \
       src->accumulator -= M_PI_M2; \
     \
+    ptr = samples; \
     if (src->accumulator < G_PI) { \
-      for (c = 0; c < channels; ++c) \
-        samples[i++] = (g##type) (src->accumulator * amp); \
+      for (c = 0; c < channels; ++c) { \
+        *ptr = (g##type) (src->accumulator * amp); \
+        ptr += channel_step; \
+      } \
     } else { \
-      for (c = 0; c < channels; ++c) \
-        samples[i++] = (g##type) ((M_PI_M2 - src->accumulator) * -amp); \
+      for (c = 0; c < channels; ++c) { \
+        *ptr = (g##type) ((M_PI_M2 - src->accumulator) * -amp); \
+        ptr += channel_step; \
+      } \
     } \
+    samples += sample_step; \
   } \
 }
 
@@ -500,29 +631,44 @@ static const ProcessFunc saw_funcs[] = {
 static void \
 gst_audio_test_src_create_triangle_##type (GstAudioTestSrc * src, g##type * samples) \
 { \
-  gint i, c, channels; \
+  gint i, c, channels, channel_step, sample_step; \
   gdouble step, amp; \
+  g##type *ptr; \
   \
   channels = GST_AUDIO_INFO_CHANNELS (&src->info); \
+  if (GST_AUDIO_INFO_LAYOUT (&src->info) == GST_AUDIO_LAYOUT_INTERLEAVED) { \
+    channel_step = 1; \
+    sample_step = channels; \
+  } else { \
+    channel_step = src->generate_samples_per_buffer; \
+    sample_step = 1; \
+  } \
   step = M_PI_M2 * src->freq / GST_AUDIO_INFO_RATE (&src->info); \
   amp = (src->volume * scale) / G_PI_2; \
   \
-  i = 0; \
-  while (i < (src->generate_samples_per_buffer * channels)) { \
+  for (i = 0; i < src->generate_samples_per_buffer; i++) { \
     src->accumulator += step; \
     if (src->accumulator >= M_PI_M2) \
       src->accumulator -= M_PI_M2; \
     \
+    ptr = samples; \
     if (src->accumulator < (G_PI_2)) { \
-      for (c = 0; c < channels; ++c) \
-        samples[i++] = (g##type) (src->accumulator * amp); \
+      for (c = 0; c < channels; ++c) { \
+        *ptr = (g##type) (src->accumulator * amp); \
+        ptr += channel_step; \
+      } \
     } else if (src->accumulator < (G_PI * 1.5)) { \
-      for (c = 0; c < channels; ++c) \
-        samples[i++] = (g##type) ((src->accumulator - G_PI) * -amp); \
+      for (c = 0; c < channels; ++c) { \
+        *ptr = (g##type) ((src->accumulator - G_PI) * -amp); \
+        ptr += channel_step; \
+      } \
     } else { \
-      for (c = 0; c < channels; ++c) \
-        samples[i++] = (g##type) ((M_PI_M2 - src->accumulator) * -amp); \
+      for (c = 0; c < channels; ++c) { \
+        *ptr = (g##type) ((M_PI_M2 - src->accumulator) * -amp); \
+        ptr += channel_step; \
+      } \
     } \
+    samples += sample_step; \
   } \
 }
 
@@ -561,14 +707,26 @@ static const ProcessFunc silence_funcs[] = {
 static void \
 gst_audio_test_src_create_white_noise_##type (GstAudioTestSrc * src, g##type * samples) \
 { \
-  gint i, c; \
+  gint i, c, channel_step, sample_step; \
+  g##type *ptr; \
   gdouble amp = (src->volume * scale); \
   gint channels = GST_AUDIO_INFO_CHANNELS (&src->info); \
   \
-  i = 0; \
-  while (i < (src->generate_samples_per_buffer * channels)) { \
-    for (c = 0; c < channels; ++c) \
-      samples[i++] = (g##type) (amp * g_rand_double_range (src->gen, -1.0, 1.0)); \
+  if (GST_AUDIO_INFO_LAYOUT (&src->info) == GST_AUDIO_LAYOUT_INTERLEAVED) { \
+    channel_step = 1; \
+    sample_step = channels; \
+  } else { \
+    channel_step = src->generate_samples_per_buffer; \
+    sample_step = 1; \
+  } \
+  \
+  for (i = 0; i < src->generate_samples_per_buffer; i++) { \
+    ptr = samples; \
+    for (c = 0; c < channels; ++c) { \
+      *ptr = (g##type) (amp * g_rand_double_range (src->gen, -1.0, 1.0)); \
+      ptr += channel_step; \
+    } \
+    samples += sample_step; \
   } \
 }
 
@@ -655,19 +813,27 @@ gst_audio_test_src_generate_pink_noise_value (GstAudioTestSrc * src)
 static void \
 gst_audio_test_src_create_pink_noise_##type (GstAudioTestSrc * src, g##type * samples) \
 { \
-  gint i, c, channels; \
+  gint i, c, channels, channel_step, sample_step; \
   gdouble amp; \
+  g##type *ptr; \
   \
   amp = src->volume * scale; \
   channels = GST_AUDIO_INFO_CHANNELS (&src->info); \
+  if (GST_AUDIO_INFO_LAYOUT (&src->info) == GST_AUDIO_LAYOUT_INTERLEAVED) { \
+    channel_step = 1; \
+    sample_step = channels; \
+  } else { \
+    channel_step = src->generate_samples_per_buffer; \
+    sample_step = 1; \
+  } \
   \
-  i = 0; \
-  while (i < (src->generate_samples_per_buffer * channels)) { \
+  for (i = 0; i < src->generate_samples_per_buffer; i++) { \
+    ptr = samples; \
     for (c = 0; c < channels; ++c) { \
-      samples[i++] = \
-        (g##type) (gst_audio_test_src_generate_pink_noise_value (src) * \
-        amp); \
+      *ptr = (g##type) (gst_audio_test_src_generate_pink_noise_value (src) * amp); \
+      ptr += channel_step; \
     } \
+    samples += sample_step; \
   } \
 }
 
@@ -684,12 +850,12 @@ static const ProcessFunc pink_noise_funcs[] = {
 };
 
 static void
-gst_audio_test_src_init_sine_table (GstAudioTestSrc * src)
+gst_audio_test_src_init_sine_table (GstAudioTestSrc * src, gboolean use_volume)
 {
   gint i;
   gdouble ang = 0.0;
   gdouble step = M_PI_M2 / 1024.0;
-  gdouble amp = src->volume;
+  gdouble amp = use_volume ? src->volume : 1.0;
 
   for (i = 0; i < 1024; i++) {
     src->wave_table[i] = sin (ang) * amp;
@@ -701,21 +867,32 @@ gst_audio_test_src_init_sine_table (GstAudioTestSrc * src)
 static void \
 gst_audio_test_src_create_sine_table_##type (GstAudioTestSrc * src, g##type * samples) \
 { \
-  gint i, c, channels; \
+  gint i, c, channels, channel_step, sample_step; \
   gdouble step, scl; \
+  g##type *ptr; \
   \
   channels = GST_AUDIO_INFO_CHANNELS (&src->info); \
+  if (GST_AUDIO_INFO_LAYOUT (&src->info) == GST_AUDIO_LAYOUT_INTERLEAVED) { \
+    channel_step = 1; \
+    sample_step = channels; \
+  } else { \
+    channel_step = src->generate_samples_per_buffer; \
+    sample_step = 1; \
+  } \
   step = M_PI_M2 * src->freq / GST_AUDIO_INFO_RATE (&src->info); \
   scl = 1024.0 / M_PI_M2; \
   \
-  i = 0; \
-  while (i < (src->generate_samples_per_buffer * channels)) { \
+  for (i = 0; i < src->generate_samples_per_buffer; i++) { \
     src->accumulator += step; \
     if (src->accumulator >= M_PI_M2) \
       src->accumulator -= M_PI_M2; \
     \
-    for (c = 0; c < channels; ++c) \
-      samples[i++] = (g##type) scale * src->wave_table[(gint) (src->accumulator * scl)]; \
+    ptr = samples; \
+    for (c = 0; c < channels; ++c) { \
+      *ptr = (g##type) scale * src->wave_table[(gint) (src->accumulator * scl)]; \
+      ptr += channel_step; \
+    } \
+    samples += sample_step; \
   } \
 }
 
@@ -731,30 +908,81 @@ static const ProcessFunc sine_table_funcs[] = {
   (ProcessFunc) gst_audio_test_src_create_sine_table_double
 };
 
+static inline gdouble
+calc_scaled_tick_volume (GstAudioTestSrc * src, gdouble scale)
+{
+  gdouble vol;
+  vol = ((src->marker_tick_period > 0)
+      && ((src->tick_counter % src->marker_tick_period) == 0))
+      ? src->marker_tick_volume : src->volume;
+  return vol * scale;
+}
+
+
 #define DEFINE_TICKS(type,scale) \
 static void \
 gst_audio_test_src_create_tick_##type (GstAudioTestSrc * src, g##type * samples) \
 { \
-  gint i, c, channels, samplerate; \
+  gint i, c, channels, samplerate, samplemod, channel_step, sample_step; \
+  gint num_nonzero_samples, num_ramp_samples, end_ramp_offset; \
   gdouble step, scl; \
+  gdouble volscale; \
+  g##type *ptr; \
   \
+  volscale = calc_scaled_tick_volume (src, scale); \
   channels = GST_AUDIO_INFO_CHANNELS (&src->info); \
+  if (GST_AUDIO_INFO_LAYOUT (&src->info) == GST_AUDIO_LAYOUT_INTERLEAVED) { \
+    channel_step = 1; \
+    sample_step = channels; \
+  } else { \
+    channel_step = src->generate_samples_per_buffer; \
+    sample_step = 1; \
+  } \
   samplerate = GST_AUDIO_INFO_RATE (&src->info); \
   step = M_PI_M2 * src->freq / samplerate; \
+  num_nonzero_samples = samplerate * src->sine_periods_per_tick / src->freq; \
   scl = 1024.0 / M_PI_M2; \
+  num_ramp_samples = src->apply_tick_ramp ? (samplerate / src->freq) : 0; \
+  end_ramp_offset = num_nonzero_samples - num_ramp_samples; \
   \
   for (i = 0; i < src->generate_samples_per_buffer; i++) { \
+    samplemod = (src->next_sample + i)%src->samples_between_ticks; \
+    \
+    ptr = samples; \
+    if (samplemod == 0) { \
+      src->accumulator = 0; \
+      src->tick_counter++; \
+      volscale = calc_scaled_tick_volume (src, scale); \
+    } else if (samplemod < num_nonzero_samples)  { \
+      gdouble ramp; \
+      if (num_ramp_samples > 0) { \
+        ramp = \
+            (samplemod < num_ramp_samples) ? (((gdouble)samplemod) / num_ramp_samples) : \
+            (samplemod >= end_ramp_offset) ? (((gdouble)(num_nonzero_samples - samplemod)) / num_ramp_samples) \
+            : 1.0; \
+        if (ramp > 1.0) \
+          ramp = 1.0; \
+        ramp *= ramp * ramp; \
+      } else \
+        ramp = 1.0; \
+      \
+      for (c = 0; c < channels; ++c) { \
+        *ptr = \
+            (g##type) volscale * ramp * src->wave_table[(gint) (src->accumulator * scl)]; \
+        ptr += channel_step; \
+      } \
+    } else { \
+      for (c = 0; c < channels; ++c) { \
+        *ptr = 0; \
+        ptr += channel_step; \
+      } \
+    } \
+    \
     src->accumulator += step; \
     if (src->accumulator >= M_PI_M2) \
       src->accumulator -= M_PI_M2; \
     \
-    if ((src->next_sample + i)%samplerate < 1600) { \
-      for (c = 0; c < channels; ++c) \
-        samples[(i * channels) + c] = (g##type) scale * src->wave_table[(gint) (src->accumulator * scl)]; \
-    } else { \
-      for (c = 0; c < channels; ++c) \
-        samples[(i * channels) + c] = 0; \
-    } \
+    samples += sample_step; \
   } \
 }
 
@@ -780,20 +1008,33 @@ static const ProcessFunc tick_funcs[] = {
 static void \
 gst_audio_test_src_create_gaussian_white_noise_##type (GstAudioTestSrc * src, g##type * samples) \
 { \
-  gint i, c; \
+  gint i, c, channel_step, sample_step; \
+  g##type *ptr; \
   gdouble amp = (src->volume * scale); \
   gint channels = GST_AUDIO_INFO_CHANNELS (&src->info); \
   \
-  for (i = 0; i < src->generate_samples_per_buffer * channels; ) { \
+  if (GST_AUDIO_INFO_LAYOUT (&src->info) == GST_AUDIO_LAYOUT_INTERLEAVED) { \
+    channel_step = 1; \
+    sample_step = channels; \
+  } else { \
+    channel_step = src->generate_samples_per_buffer; \
+    sample_step = 1; \
+  } \
+  \
+  for (i = 0; i < src->generate_samples_per_buffer; i++) { \
+    ptr = samples; \
     for (c = 0; c < channels; ++c) { \
       gdouble mag = sqrt (-2 * log (1.0 - g_rand_double (src->gen))); \
       gdouble phs = g_rand_double_range (src->gen, 0.0, M_PI_M2); \
       \
-      samples[i++] = (g##type) (amp * mag * cos (phs)); \
+      *ptr = (g##type) (amp * mag * cos (phs)); \
+      ptr += channel_step; \
       if (++c >= channels) \
         break; \
-      samples[i++] = (g##type) (amp * mag * sin (phs)); \
+      *ptr = (g##type) (amp * mag * sin (phs)); \
+      ptr += channel_step; \
     } \
+    samples += sample_step; \
   } \
 }
 
@@ -820,12 +1061,22 @@ static const ProcessFunc gaussian_white_noise_funcs[] = {
 static void \
 gst_audio_test_src_create_red_noise_##type (GstAudioTestSrc * src, g##type * samples) \
 { \
-  gint i, c; \
+  gint i, c, channel_step, sample_step; \
+  g##type *ptr; \
   gdouble amp = (src->volume * scale); \
   gdouble state = src->red.state; \
   gint channels = GST_AUDIO_INFO_CHANNELS (&src->info); \
   \
-  for (i = 0; i < src->generate_samples_per_buffer * channels; ) { \
+  if (GST_AUDIO_INFO_LAYOUT (&src->info) == GST_AUDIO_LAYOUT_INTERLEAVED) { \
+    channel_step = 1; \
+    sample_step = channels; \
+  } else { \
+    channel_step = src->generate_samples_per_buffer; \
+    sample_step = 1; \
+  } \
+  \
+  for (i = 0; i < src->generate_samples_per_buffer; i++) { \
+    ptr = samples; \
     for (c = 0; c < channels; ++c) { \
       while (TRUE) { \
         gdouble r = g_rand_double_range (src->gen, -1.0, 1.0); \
@@ -833,8 +1084,10 @@ gst_audio_test_src_create_red_noise_##type (GstAudioTestSrc * src, g##type * sam
         if (state < -8.0f || state > 8.0f) state -= r; \
         else break; \
       } \
-      samples[i++] = (g##type) (amp * state * 0.0625f); /* /16.0 */ \
+      *ptr = (g##type) (amp * state * 0.0625f); /* /16.0 */ \
+      ptr += channel_step; \
     } \
+    samples += sample_step; \
   } \
   src->red.state = state; \
 }
@@ -857,16 +1110,28 @@ static const ProcessFunc red_noise_funcs[] = {
 static void \
 gst_audio_test_src_create_blue_noise_##type (GstAudioTestSrc * src, g##type * samples) \
 { \
-  gint i, c; \
+  gint i, c, channel_step, sample_step; \
   static gdouble flip=1.0; \
   gint channels = GST_AUDIO_INFO_CHANNELS (&src->info); \
+  g##type *ptr; \
+  \
+  if (GST_AUDIO_INFO_LAYOUT (&src->info) == GST_AUDIO_LAYOUT_INTERLEAVED) { \
+    channel_step = 1; \
+    sample_step = channels; \
+  } else { \
+    channel_step = src->generate_samples_per_buffer; \
+    sample_step = 1; \
+  } \
   \
   gst_audio_test_src_create_pink_noise_##type (src, samples); \
-  for (i = 0; i < src->generate_samples_per_buffer * channels; ) { \
+  for (i = 0; i < src->generate_samples_per_buffer; i++) { \
+    ptr = samples; \
     for (c = 0; c < channels; ++c) { \
-      samples[i++] *= flip; \
+      *ptr *= flip; \
+      ptr += channel_step; \
     } \
     flip *= -1.0; \
+    samples += sample_step; \
   } \
 }
 
@@ -889,16 +1154,28 @@ static const ProcessFunc blue_noise_funcs[] = {
 static void \
 gst_audio_test_src_create_violet_noise_##type (GstAudioTestSrc * src, g##type * samples) \
 { \
-  gint i, c; \
+  gint i, c, channel_step, sample_step; \
   static gdouble flip=1.0; \
   gint channels = GST_AUDIO_INFO_CHANNELS (&src->info); \
+  g##type *ptr; \
+  \
+  if (GST_AUDIO_INFO_LAYOUT (&src->info) == GST_AUDIO_LAYOUT_INTERLEAVED) { \
+    channel_step = 1; \
+    sample_step = channels; \
+  } else { \
+    channel_step = src->generate_samples_per_buffer; \
+    sample_step = 1; \
+  } \
   \
   gst_audio_test_src_create_red_noise_##type (src, samples); \
-  for (i = 0; i < src->generate_samples_per_buffer * channels; ) { \
+  for (i = 0; i < src->generate_samples_per_buffer; i++) { \
+    ptr = samples; \
     for (c = 0; c < channels; ++c) { \
-      samples[i++] *= flip; \
+      *ptr *= flip; \
+      ptr += channel_step; \
     } \
     flip *= -1.0; \
+    samples += sample_step; \
   } \
 }
 
@@ -991,12 +1268,15 @@ gst_audio_test_src_change_wave (GstAudioTestSrc * src)
       src->process = pink_noise_funcs[idx];
       break;
     case GST_AUDIO_TEST_SRC_WAVE_SINE_TAB:
-      gst_audio_test_src_init_sine_table (src);
+      gst_audio_test_src_init_sine_table (src, TRUE);
       src->process = sine_table_funcs[idx];
       break;
     case GST_AUDIO_TEST_SRC_WAVE_TICKS:
-      gst_audio_test_src_init_sine_table (src);
+      gst_audio_test_src_init_sine_table (src, FALSE);
       src->process = tick_funcs[idx];
+      src->samples_between_ticks =
+          gst_util_uint64_scale_int (src->tick_interval,
+          GST_AUDIO_INFO_RATE (&(src->info)), GST_SECOND);
       break;
     case GST_AUDIO_TEST_SRC_WAVE_GAUSSIAN_WHITE_NOISE:
       if (!(src->gen))
@@ -1036,7 +1316,7 @@ gst_audio_test_src_change_volume (GstAudioTestSrc * src)
 {
   switch (src->wave) {
     case GST_AUDIO_TEST_SRC_WAVE_SINE_TAB:
-      gst_audio_test_src_init_sine_table (src);
+      gst_audio_test_src_init_sine_table (src, TRUE);
       break;
     default:
       break;
@@ -1078,6 +1358,7 @@ gst_audio_test_src_start (GstBaseSrc * basesrc)
   src->eos_reached = FALSE;
   src->tags_pushed = FALSE;
   src->accumulator = 0;
+  src->tick_counter = 0;
 
   return TRUE;
 }
@@ -1123,21 +1404,16 @@ gst_audio_test_src_do_seek (GstBaseSrc * basesrc, GstSegment * segment)
 
   src->next_sample = next_sample;
 
-  if (!src->reverse) {
-    if (GST_CLOCK_TIME_IS_VALID (segment->start)) {
-      segment->time = segment->start;
-    }
-  } else {
-    if (GST_CLOCK_TIME_IS_VALID (segment->stop)) {
-      segment->time = segment->stop;
-    }
-  }
-
-  if (GST_CLOCK_TIME_IS_VALID (segment->stop)) {
+  if (segment->rate > 0 && GST_CLOCK_TIME_IS_VALID (segment->stop)) {
     time = segment->stop;
     src->sample_stop =
         gst_util_uint64_scale_round (time, samplerate, GST_SECOND);
     src->check_seek_stop = TRUE;
+  } else if (segment->rate < 0) {
+    time = segment->start;
+    src->sample_stop =
+        gst_util_uint64_scale_round (time, samplerate, GST_SECOND);
+    src->check_seek_stop = TRUE;
   } else {
     src->check_seek_stop = FALSE;
   }
@@ -1212,7 +1488,7 @@ gst_audio_test_src_fill (GstBaseSrc * basesrc, guint64 offset,
   }
 
   /* check for eos */
-  if (src->check_seek_stop &&
+  if (src->check_seek_stop && !src->reverse &&
       (src->sample_stop > src->next_sample) &&
       (src->sample_stop < src->next_sample + samples)
       ) {
@@ -1220,6 +1496,13 @@ gst_audio_test_src_fill (GstBaseSrc * basesrc, guint64 offset,
     src->generate_samples_per_buffer = src->sample_stop - src->next_sample;
     next_sample = src->sample_stop;
     src->eos_reached = TRUE;
+  } else if (src->check_seek_stop && src->reverse &&
+      (src->sample_stop > src->next_sample)
+      ) {
+    /* calculate only partial buffer */
+    src->generate_samples_per_buffer = src->sample_stop - src->next_sample;
+    next_sample = src->sample_stop;
+    src->eos_reached = TRUE;
   } else {
     /* calculate full buffer */
     src->generate_samples_per_buffer = samples;
@@ -1283,6 +1566,11 @@ gst_audio_test_src_fill (GstBaseSrc * basesrc, guint64 offset,
     GST_BUFFER_FLAG_SET (buffer, GST_BUFFER_FLAG_GAP);
   }
 
+  if (GST_AUDIO_INFO_LAYOUT (&src->info) == GST_AUDIO_LAYOUT_NON_INTERLEAVED) {
+    gst_buffer_add_audio_meta (buffer, &src->info,
+        src->generate_samples_per_buffer, NULL);
+  }
+
   return GST_FLOW_OK;
 }
 
@@ -1315,6 +1603,21 @@ gst_audio_test_src_set_property (GObject * object, guint prop_id,
     case PROP_TIMESTAMP_OFFSET:
       src->timestamp_offset = g_value_get_int64 (value);
       break;
+    case PROP_SINE_PERIODS_PER_TICK:
+      src->sine_periods_per_tick = g_value_get_uint (value);
+      break;
+    case PROP_TICK_INTERVAL:
+      src->tick_interval = g_value_get_uint64 (value);
+      break;
+    case PROP_MARKER_TICK_PERIOD:
+      src->marker_tick_period = g_value_get_uint (value);
+      break;
+    case PROP_MARKER_TICK_VOLUME:
+      src->marker_tick_volume = g_value_get_double (value);
+      break;
+    case PROP_APPLY_TICK_RAMP:
+      src->apply_tick_ramp = g_value_get_boolean (value);
+      break;
     case PROP_CAN_ACTIVATE_PUSH:
       GST_BASE_SRC (src)->can_activate_push = g_value_get_boolean (value);
       break;
@@ -1352,6 +1655,21 @@ gst_audio_test_src_get_property (GObject * object, guint prop_id,
     case PROP_TIMESTAMP_OFFSET:
       g_value_set_int64 (value, src->timestamp_offset);
       break;
+    case PROP_SINE_PERIODS_PER_TICK:
+      g_value_set_uint (value, src->sine_periods_per_tick);
+      break;
+    case PROP_TICK_INTERVAL:
+      g_value_set_uint64 (value, src->tick_interval);
+      break;
+    case PROP_MARKER_TICK_PERIOD:
+      g_value_set_uint (value, src->marker_tick_period);
+      break;
+    case PROP_MARKER_TICK_VOLUME:
+      g_value_set_double (value, src->marker_tick_volume);
+      break;
+    case PROP_APPLY_TICK_RAMP:
+      g_value_set_boolean (value, src->apply_tick_ramp);
+      break;
     case PROP_CAN_ACTIVATE_PUSH:
       g_value_set_boolean (value, GST_BASE_SRC (src)->can_activate_push);
       break;
