@@ -70,8 +70,8 @@ typedef struct _GstURISourceBinClass GstURISourceBinClass;
 typedef struct _ChildSrcPadInfo ChildSrcPadInfo;
 typedef struct _OutputSlotInfo OutputSlotInfo;
 
-#define GST_URI_SOURCE_BIN_LOCK(dec) (g_mutex_lock(&((GstURISourceBin*)(dec))->lock))
-#define GST_URI_SOURCE_BIN_UNLOCK(dec) (g_mutex_unlock(&((GstURISourceBin*)(dec))->lock))
+#define GST_URI_SOURCE_BIN_LOCK(urisrc) (g_mutex_lock(&((GstURISourceBin*)(urisrc))->lock))
+#define GST_URI_SOURCE_BIN_UNLOCK(urisrc) (g_mutex_unlock(&((GstURISourceBin*)(urisrc))->lock))
 
 #define BUFFERING_LOCK(ubin) G_STMT_START {				\
     GST_LOG_OBJECT (ubin,						\
@@ -111,6 +111,8 @@ struct _OutputSlotInfo
   GstPad *sinkpad;              /* Sink pad of the queue eleemnt */
   GstPad *srcpad;               /* Output ghost pad */
   gboolean is_eos;              /* Did EOS get fed into the buffering element */
+
+  gulong bitrate_changed_id;    /* queue bitrate changed notification */
 };
 
 /**
@@ -138,6 +140,8 @@ struct _GstURISourceBin
   guint buffer_size;            /* When buffering, buffer size (bytes) */
   gboolean download;
   gboolean use_buffering;
+  gdouble low_watermark;
+  gdouble high_watermark;
 
   GstElement *source;
   GList *typefinds;             /* list of typefind element */
@@ -145,7 +149,6 @@ struct _GstURISourceBin
   GstElement *demuxer;          /* Adaptive demuxer if any */
   GSList *out_slots;
 
-  GHashTable *streams;
   guint numpads;
 
   /* for dynamic sources */
@@ -203,6 +206,14 @@ enum
 #define DEFAULT_DOWNLOAD            FALSE
 #define DEFAULT_USE_BUFFERING       TRUE
 #define DEFAULT_RING_BUFFER_MAX_SIZE 0
+#define DEFAULT_LOW_WATERMARK       0.01
+#define DEFAULT_HIGH_WATERMARK      0.99
+
+#define ACTUAL_DEFAULT_BUFFER_SIZE  10 * 1024 * 1024    /* The value used for byte limits when buffer-size == -1 */
+#define ACTUAL_DEFAULT_BUFFER_DURATION  5 * GST_SECOND  /* The value used for time limits when buffer-duration == -1 */
+
+#define GET_BUFFER_SIZE(u) ((u)->buffer_size == -1 ? ACTUAL_DEFAULT_BUFFER_SIZE : (u)->buffer_size)
+#define GET_BUFFER_DURATION(u) ((u)->buffer_duration == -1 ? ACTUAL_DEFAULT_BUFFER_DURATION : (u)->buffer_duration)
 
 #define DEFAULT_CAPS (gst_static_caps_get (&default_raw_caps))
 enum
@@ -215,10 +226,28 @@ enum
   PROP_BUFFER_DURATION,
   PROP_DOWNLOAD,
   PROP_USE_BUFFERING,
-  PROP_RING_BUFFER_MAX_SIZE
+  PROP_RING_BUFFER_MAX_SIZE,
+  PROP_LOW_WATERMARK,
+  PROP_HIGH_WATERMARK,
+  PROP_STATISTICS,
 };
 
-static void post_missing_plugin_error (GstElement * dec,
+#define CUSTOM_EOS_QUARK _custom_eos_quark_get ()
+#define CUSTOM_EOS_QUARK_DATA "custom-eos"
+static GQuark
+_custom_eos_quark_get (void)
+{
+  static gsize g_quark;
+
+  if (g_once_init_enter (&g_quark)) {
+    gsize quark =
+        (gsize) g_quark_from_static_string ("urisourcebin-custom-eos");
+    g_once_init_leave (&g_quark, quark);
+  }
+  return g_quark;
+}
+
+static void post_missing_plugin_error (GstElement * urisrc,
     const gchar * element_name);
 
 static guint gst_uri_source_bin_signals[LAST_SIGNAL] = { 0 };
@@ -250,6 +279,9 @@ static void free_output_slot_async (GstURISourceBin * urisrc,
 static GstPad *create_output_pad (GstURISourceBin * urisrc, GstPad * pad);
 static void remove_buffering_msgs (GstURISourceBin * bin, GstObject * src);
 
+static void update_queue_values (GstURISourceBin * urisrc);
+static GstStructure *get_queue_statistics (GstURISourceBin * urisrc);
+
 static void
 gst_uri_source_bin_class_init (GstURISourceBinClass * klass)
 {
@@ -330,6 +362,49 @@ gst_uri_source_bin_class_init (GstURISourceBinClass * klass)
           0, G_MAXUINT, DEFAULT_RING_BUFFER_MAX_SIZE,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
+  /**
+   * GstURISourceBin::low-watermark
+   *
+   * Proportion of the queue size (either in bytes or time) for buffering
+   * to restart when crossed from above.  Only used if use-buffering is TRUE.
+   */
+  g_object_class_install_property (gobject_class, PROP_LOW_WATERMARK,
+      g_param_spec_double ("low-watermark", "Low watermark",
+          "Low threshold for buffering to start. Only used if use-buffering is True",
+          0.0, 1.0, DEFAULT_LOW_WATERMARK,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+  /**
+   * GstURISourceBin::high-watermark
+   *
+   * Proportion of the queue size (either in bytes or time) to complete
+   * buffering.  Only used if use-buffering is TRUE.
+   */
+  g_object_class_install_property (gobject_class, PROP_HIGH_WATERMARK,
+      g_param_spec_double ("high-watermark", "High watermark",
+          "High threshold for buffering to finish. Only used if use-buffering is True",
+          0.0, 1.0, DEFAULT_HIGH_WATERMARK,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+  /**
+   * GstURISourceBin::statistics
+   *
+   * A GStructure containing the following values based on the values from
+   * all the queue's contained in this urisourcebin.
+   *
+   *  "minimum-byte-level"  G_TYPE_UINT               Minimum of the current byte levels
+   *  "maximum-byte-level"  G_TYPE_UINT               Maximum of the current byte levels
+   *  "average-byte-level"  G_TYPE_UINT               Average of the current byte levels
+   *  "minimum-time-level"  G_TYPE_UINT64             Minimum of the current time levels
+   *  "maximum-time-level"  G_TYPE_UINT64             Maximum of the current time levels
+   *  "average-time-level"  G_TYPE_UINT64             Average of the current time levels
+   */
+  g_object_class_install_property (gobject_class, PROP_STATISTICS,
+      g_param_spec_boxed ("statistics", "Queue Statistics",
+          "A set of statistics over all the queue-like elements contained in "
+          "this element", GST_TYPE_STRUCTURE,
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
   /**
    * GstURISourceBin::drained:
    *
@@ -404,8 +479,11 @@ gst_uri_source_bin_init (GstURISourceBin * urisrc)
   urisrc->use_buffering = DEFAULT_USE_BUFFERING;
   urisrc->ring_buffer_max_size = DEFAULT_RING_BUFFER_MAX_SIZE;
   urisrc->last_buffering_pct = -1;
+  urisrc->low_watermark = DEFAULT_LOW_WATERMARK;
+  urisrc->high_watermark = DEFAULT_HIGH_WATERMARK;
 
-  GST_OBJECT_FLAG_SET (urisrc, GST_ELEMENT_FLAG_SOURCE);
+  GST_OBJECT_FLAG_SET (urisrc,
+      GST_ELEMENT_FLAG_SOURCE | GST_BIN_FLAG_STREAMS_AWARE);
   gst_bin_set_suppressed_flags (GST_BIN (urisrc),
       GST_ELEMENT_FLAG_SOURCE | GST_ELEMENT_FLAG_SINK);
 }
@@ -431,34 +509,44 @@ static void
 gst_uri_source_bin_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
 {
-  GstURISourceBin *dec = GST_URI_SOURCE_BIN (object);
+  GstURISourceBin *urisrc = GST_URI_SOURCE_BIN (object);
 
   switch (prop_id) {
     case PROP_URI:
-      GST_OBJECT_LOCK (dec);
-      g_free (dec->uri);
-      dec->uri = g_value_dup_string (value);
-      GST_OBJECT_UNLOCK (dec);
+      GST_OBJECT_LOCK (urisrc);
+      g_free (urisrc->uri);
+      urisrc->uri = g_value_dup_string (value);
+      GST_OBJECT_UNLOCK (urisrc);
       break;
     case PROP_CONNECTION_SPEED:
-      GST_OBJECT_LOCK (dec);
-      dec->connection_speed = g_value_get_uint64 (value) * 1000;
-      GST_OBJECT_UNLOCK (dec);
+      GST_OBJECT_LOCK (urisrc);
+      urisrc->connection_speed = g_value_get_uint64 (value) * 1000;
+      GST_OBJECT_UNLOCK (urisrc);
       break;
     case PROP_BUFFER_SIZE:
-      dec->buffer_size = g_value_get_int (value);
+      urisrc->buffer_size = g_value_get_int (value);
+      update_queue_values (urisrc);
       break;
     case PROP_BUFFER_DURATION:
-      dec->buffer_duration = g_value_get_int64 (value);
+      urisrc->buffer_duration = g_value_get_int64 (value);
+      update_queue_values (urisrc);
       break;
     case PROP_DOWNLOAD:
-      dec->download = g_value_get_boolean (value);
+      urisrc->download = g_value_get_boolean (value);
       break;
     case PROP_USE_BUFFERING:
-      dec->use_buffering = g_value_get_boolean (value);
+      urisrc->use_buffering = g_value_get_boolean (value);
       break;
     case PROP_RING_BUFFER_MAX_SIZE:
-      dec->ring_buffer_max_size = g_value_get_uint64 (value);
+      urisrc->ring_buffer_max_size = g_value_get_uint64 (value);
+      break;
+    case PROP_LOW_WATERMARK:
+      urisrc->low_watermark = g_value_get_double (value);
+      update_queue_values (urisrc);
+      break;
+    case PROP_HIGH_WATERMARK:
+      urisrc->high_watermark = g_value_get_double (value);
+      update_queue_values (urisrc);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -470,42 +558,51 @@ static void
 gst_uri_source_bin_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec)
 {
-  GstURISourceBin *dec = GST_URI_SOURCE_BIN (object);
+  GstURISourceBin *urisrc = GST_URI_SOURCE_BIN (object);
 
   switch (prop_id) {
     case PROP_URI:
-      GST_OBJECT_LOCK (dec);
-      g_value_set_string (value, dec->uri);
-      GST_OBJECT_UNLOCK (dec);
+      GST_OBJECT_LOCK (urisrc);
+      g_value_set_string (value, urisrc->uri);
+      GST_OBJECT_UNLOCK (urisrc);
       break;
     case PROP_SOURCE:
-      GST_OBJECT_LOCK (dec);
-      g_value_set_object (value, dec->source);
-      GST_OBJECT_UNLOCK (dec);
+      GST_OBJECT_LOCK (urisrc);
+      g_value_set_object (value, urisrc->source);
+      GST_OBJECT_UNLOCK (urisrc);
       break;
     case PROP_CONNECTION_SPEED:
-      GST_OBJECT_LOCK (dec);
-      g_value_set_uint64 (value, dec->connection_speed / 1000);
-      GST_OBJECT_UNLOCK (dec);
+      GST_OBJECT_LOCK (urisrc);
+      g_value_set_uint64 (value, urisrc->connection_speed / 1000);
+      GST_OBJECT_UNLOCK (urisrc);
       break;
     case PROP_BUFFER_SIZE:
-      GST_OBJECT_LOCK (dec);
-      g_value_set_int (value, dec->buffer_size);
-      GST_OBJECT_UNLOCK (dec);
+      GST_OBJECT_LOCK (urisrc);
+      g_value_set_int (value, urisrc->buffer_size);
+      GST_OBJECT_UNLOCK (urisrc);
       break;
     case PROP_BUFFER_DURATION:
-      GST_OBJECT_LOCK (dec);
-      g_value_set_int64 (value, dec->buffer_duration);
-      GST_OBJECT_UNLOCK (dec);
+      GST_OBJECT_LOCK (urisrc);
+      g_value_set_int64 (value, urisrc->buffer_duration);
+      GST_OBJECT_UNLOCK (urisrc);
       break;
     case PROP_DOWNLOAD:
-      g_value_set_boolean (value, dec->download);
+      g_value_set_boolean (value, urisrc->download);
       break;
     case PROP_USE_BUFFERING:
-      g_value_set_boolean (value, dec->use_buffering);
+      g_value_set_boolean (value, urisrc->use_buffering);
       break;
     case PROP_RING_BUFFER_MAX_SIZE:
-      g_value_set_uint64 (value, dec->ring_buffer_max_size);
+      g_value_set_uint64 (value, urisrc->ring_buffer_max_size);
+      break;
+    case PROP_LOW_WATERMARK:
+      g_value_set_double (value, urisrc->low_watermark);
+      break;
+    case PROP_HIGH_WATERMARK:
+      g_value_set_double (value, urisrc->high_watermark);
+      break;
+    case PROP_STATISTICS:
+      g_value_take_boxed (value, get_queue_statistics (urisrc));
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -513,11 +610,6 @@ gst_uri_source_bin_get_property (GObject * object, guint prop_id,
   }
 }
 
-
-#define DEFAULT_QUEUE_SIZE          (3 * GST_SECOND)
-#define DEFAULT_QUEUE_MIN_THRESHOLD ((DEFAULT_QUEUE_SIZE * 30) / 100)
-#define DEFAULT_QUEUE_THRESHOLD     ((DEFAULT_QUEUE_SIZE * 95) / 100)
-
 static gboolean
 copy_sticky_events (GstPad * pad, GstEvent ** event, gpointer user_data)
 {
@@ -747,7 +839,6 @@ demux_pad_events (GstPad * pad, GstPadProbeInfo * info, gpointer user_data)
   switch (GST_EVENT_TYPE (ev)) {
     case GST_EVENT_EOS:
     {
-      GstStructure *s;
       gboolean all_streams_eos;
 
       GST_LOG_OBJECT (urisrc, "EOS on pad %" GST_PTR_FORMAT, pad);
@@ -771,11 +862,8 @@ demux_pad_events (GstPad * pad, GstPadProbeInfo * info, gpointer user_data)
           GST_OBJECT_CAST (child_info->output_slot->queue));
 
       /* Mark this custom EOS */
-      ev = gst_event_make_writable (ev);
-      GST_PAD_PROBE_INFO_DATA (info) = ev;
-      s = gst_event_writable_structure (ev);
-      gst_structure_set (s, "urisourcebin-custom-eos", G_TYPE_BOOLEAN, TRUE,
-          NULL);
+      gst_mini_object_set_qdata (GST_MINI_OBJECT_CAST (ev), CUSTOM_EOS_QUARK,
+          (gchar *) CUSTOM_EOS_QUARK_DATA, NULL);
       if (all_streams_eos) {
         GST_DEBUG_OBJECT (urisrc, "POSTING ABOUT TO FINISH");
         g_signal_emit (urisrc,
@@ -828,6 +916,131 @@ pre_queue_event_probe (GstPad * pad, GstPadProbeInfo * info, gpointer user_data)
   return ret;
 }
 
+static GstStructure *
+get_queue_statistics (GstURISourceBin * urisrc)
+{
+  GstStructure *ret = NULL;
+  guint min_byte_level = 0, max_byte_level = 0;
+  guint64 min_time_level = 0, max_time_level = 0;
+  gdouble avg_byte_level = 0., avg_time_level = 0.;
+  guint i = 0;
+  GSList *cur;
+
+  GST_URI_SOURCE_BIN_LOCK (urisrc);
+
+  for (cur = urisrc->out_slots; cur != NULL; cur = g_slist_next (cur)) {
+    OutputSlotInfo *slot = (OutputSlotInfo *) (cur->data);
+    guint byte_limit = 0;
+    guint64 time_limit = 0;
+
+    g_object_get (slot->queue, "current-level-bytes", &byte_limit,
+        "current-level-time", &time_limit, NULL);
+
+    if (byte_limit < min_byte_level)
+      min_byte_level = byte_limit;
+    if (byte_limit > max_byte_level)
+      max_byte_level = byte_limit;
+    avg_byte_level = (avg_byte_level * i + byte_limit) / (gdouble) (i + 1);
+
+    if (time_limit < min_time_level)
+      min_time_level = time_limit;
+    if (time_limit > max_time_level)
+      max_time_level = time_limit;
+    avg_time_level = (avg_time_level * i + time_limit) / (gdouble) (i + 1);
+
+    i++;
+  }
+  GST_URI_SOURCE_BIN_UNLOCK (urisrc);
+
+  ret = gst_structure_new ("application/x-urisourcebin-stats",
+      "minimum-byte-level", G_TYPE_UINT, (guint) min_byte_level,
+      "maximum-byte-level", G_TYPE_UINT, (guint) max_byte_level,
+      "average-byte-level", G_TYPE_UINT, (guint) avg_byte_level,
+      "minimum-time-level", G_TYPE_UINT64, (guint64) min_time_level,
+      "maximum-time-level", G_TYPE_UINT64, (guint64) max_time_level,
+      "average-time-level", G_TYPE_UINT64, (guint64) avg_time_level, NULL);
+
+  return ret;
+}
+
+static void
+update_queue_values (GstURISourceBin * urisrc)
+{
+  gint64 duration;
+  guint buffer_size;
+  gdouble low_watermark, high_watermark;
+  guint64 cumulative_bitrate = 0;
+  GSList *cur;
+
+  GST_URI_SOURCE_BIN_LOCK (urisrc);
+  duration = GET_BUFFER_DURATION (urisrc);
+  buffer_size = GET_BUFFER_SIZE (urisrc);
+  low_watermark = urisrc->low_watermark;
+  high_watermark = urisrc->high_watermark;
+
+  for (cur = urisrc->out_slots; cur != NULL; cur = g_slist_next (cur)) {
+    OutputSlotInfo *slot = (OutputSlotInfo *) (cur->data);
+    guint64 bitrate = 0;
+
+    if (g_object_class_find_property (G_OBJECT_GET_CLASS (slot->queue),
+            "bitrate")) {
+      g_object_get (G_OBJECT (slot->queue), "bitrate", &bitrate, NULL);
+    }
+
+    if (bitrate > 0)
+      cumulative_bitrate += bitrate;
+    else {
+      GST_TRACE_OBJECT (urisrc, "Unknown bitrate detected from %" GST_PTR_FORMAT
+          ", resetting all bitrates", slot->queue);
+      cumulative_bitrate = 0;
+      break;
+    }
+  }
+
+  GST_DEBUG_OBJECT (urisrc, "recalculating queue limits with cumulative "
+      "bitrate %" G_GUINT64_FORMAT ", buffer size %u, buffer duration %"
+      G_GINT64_FORMAT, cumulative_bitrate, buffer_size, duration);
+
+  for (cur = urisrc->out_slots; cur != NULL; cur = g_slist_next (cur)) {
+    OutputSlotInfo *slot = (OutputSlotInfo *) (cur->data);
+    guint byte_limit;
+
+    if (cumulative_bitrate > 0
+        && g_object_class_find_property (G_OBJECT_GET_CLASS (slot->queue),
+            "bitrate")) {
+      guint64 bitrate;
+      g_object_get (G_OBJECT (slot->queue), "bitrate", &bitrate, NULL);
+      byte_limit =
+          gst_util_uint64_scale (buffer_size, bitrate, cumulative_bitrate);
+    } else {
+      /* if not all queue's have valid bitrates, use the buffer-size as the
+       * limit */
+      byte_limit = buffer_size;
+    }
+
+    GST_DEBUG_OBJECT (urisrc,
+        "calculated new limits for queue-like element %" GST_PTR_FORMAT
+        ", bytes:%u, time:%" G_GUINT64_FORMAT
+        ", low-watermark:%f, high-watermark:%f",
+        slot->queue, byte_limit, (guint64) duration, low_watermark,
+        high_watermark);
+    g_object_set (G_OBJECT (slot->queue), "max-size-bytes", byte_limit,
+        "max-size-time", (guint64) duration, "low-watermark", low_watermark,
+        "high-watermark", high_watermark, NULL);
+  }
+  GST_URI_SOURCE_BIN_UNLOCK (urisrc);
+}
+
+static void
+on_queue_bitrate_changed (GstElement * queue, GParamSpec * pspec,
+    gpointer user_data)
+{
+  GstURISourceBin *urisrc = GST_URI_SOURCE_BIN (user_data);
+
+  gst_element_call_async (GST_ELEMENT (urisrc),
+      (GstElementCallAsyncFunc) update_queue_values, NULL, NULL);
+}
+
 /* Called with lock held */
 static OutputSlotInfo *
 get_output_slot (GstURISourceBin * urisrc, gboolean do_download,
@@ -875,6 +1088,10 @@ get_output_slot (GstURISourceBin * urisrc, gboolean do_download,
   /* Set the slot onto the queue (needed in buffering msg handling) */
   g_object_set_data (G_OBJECT (queue), "urisourcebin.slotinfo", slot);
 
+  slot->bitrate_changed_id =
+      g_signal_connect (G_OBJECT (queue), "notify::bitrate",
+      (GCallback) on_queue_bitrate_changed, urisrc);
+
   if (do_download) {
     gchar *temp_template, *filename;
     const gchar *tmp_dir, *prgname;
@@ -915,13 +1132,14 @@ get_output_slot (GstURISourceBin * urisrc, gboolean do_download,
      * Start playback when the queue is 60% full, leaving a bit more room
      * for upstream to push more without getting bursty */
     g_object_set (queue, "low-percent", 1, "high-percent", 60, NULL);
+
+    g_object_set (queue, "low-watermark", urisrc->low_watermark,
+        "high-watermark", urisrc->high_watermark, NULL);
   }
 
-  /* If buffer size or duration are set, set them on the element */
-  if (urisrc->buffer_size != -1)
-    g_object_set (queue, "max-size-bytes", urisrc->buffer_size, NULL);
-  if (urisrc->buffer_duration != -1)
-    g_object_set (queue, "max-size-time", urisrc->buffer_duration, NULL);
+  /* set the necessary limits on the queue-like elements */
+  g_object_set (queue, "max-size-bytes", GET_BUFFER_SIZE (urisrc),
+      "max-size-time", (guint64) GET_BUFFER_DURATION (urisrc), NULL);
 #if 0
   /* Disabled because this makes initial startup slower for radio streams */
   else {
@@ -965,12 +1183,16 @@ source_pad_event_probe (GstPad * pad, GstPadProbeInfo * info,
 
   GST_LOG_OBJECT (pad, "%s, urisrc %p", GST_EVENT_TYPE_NAME (event), event);
 
-  if (GST_EVENT_TYPE (event) == GST_EVENT_EOS && gst_event_get_structure (event)
-      && gst_structure_has_field (gst_event_get_structure (event),
-          "urisourcebin-custom-eos")) {
+  if (GST_EVENT_TYPE (event) == GST_EVENT_EOS &&
+      gst_mini_object_get_qdata (GST_MINI_OBJECT_CAST (event),
+          CUSTOM_EOS_QUARK)) {
     OutputSlotInfo *slot;
     GST_DEBUG_OBJECT (pad, "we received EOS");
 
+    /* remove custom-eos */
+    gst_mini_object_set_qdata (GST_MINI_OBJECT_CAST (event), CUSTOM_EOS_QUARK,
+        NULL, NULL);
+
     GST_URI_SOURCE_BIN_LOCK (urisrc);
 
     slot = g_object_get_data (G_OBJECT (pad), "urisourcebin.slotinfo");
@@ -1364,14 +1586,14 @@ has_all_raw_caps (GstPad * pad, GstCaps * rawcaps, gboolean * all_raw)
 }
 
 static void
-post_missing_plugin_error (GstElement * dec, const gchar * element_name)
+post_missing_plugin_error (GstElement * urisrc, const gchar * element_name)
 {
   GstMessage *msg;
 
-  msg = gst_missing_element_message_new (dec, element_name);
-  gst_element_post_message (dec, msg);
+  msg = gst_missing_element_message_new (urisrc, element_name);
+  gst_element_post_message (urisrc, msg);
 
-  GST_ELEMENT_ERROR (dec, CORE, MISSING_PLUGIN,
+  GST_ELEMENT_ERROR (urisrc, CORE, MISSING_PLUGIN,
       (_("Missing element '%s' - check your GStreamer installation."),
           element_name), (NULL));
 }
@@ -1454,13 +1676,16 @@ analyse_source (GstURISourceBin * urisrc, gboolean * is_raw,
             gst_object_unref (pad);
             pad = slot->srcpad;
           } else {
-            pad = create_output_pad (urisrc, pad);
+            GstPad *tmppad = create_output_pad (urisrc, pad);
+            gst_object_unref (pad);
+
+            pad = tmppad;
           }
           GST_URI_SOURCE_BIN_UNLOCK (urisrc);
           expose_output_pad (urisrc, pad);
+        } else {
           gst_object_unref (pad);
         }
-        gst_object_unref (pad);
         g_value_reset (&item);
         break;
     }
@@ -1776,6 +2001,10 @@ free_output_slot (OutputSlotInfo * slot, GstURISourceBin * urisrc)
 {
   GST_DEBUG_OBJECT (urisrc, "removing old queue element and freeing slot %p",
       slot);
+  if (slot->bitrate_changed_id > 0)
+    g_signal_handler_disconnect (slot->queue, slot->bitrate_changed_id);
+  slot->bitrate_changed_id = 0;
+
   gst_element_set_locked_state (slot->queue, TRUE);
   gst_element_set_state (slot->queue, GST_STATE_NULL);
   gst_bin_remove (GST_BIN_CAST (urisrc), slot->queue);
@@ -2029,7 +2258,7 @@ value_list_append_structure_list (GValue * list_val, GstStructure ** first,
  * want to pick a different 'best' location depending on the required
  * bitrates and the connection speed */
 static GstMessage *
-handle_redirect_message (GstURISourceBin * dec, GstMessage * msg)
+handle_redirect_message (GstURISourceBin * urisrc, GstMessage * msg)
 {
   const GValue *locations_list, *location_val;
   GstMessage *new_msg;
@@ -2039,12 +2268,12 @@ handle_redirect_message (GstURISourceBin * dec, GstMessage * msg)
   guint size, i;
   const GstStructure *structure;
 
-  GST_DEBUG_OBJECT (dec, "redirect message: %" GST_PTR_FORMAT, msg);
-  GST_DEBUG_OBJECT (dec, "connection speed: %" G_GUINT64_FORMAT,
-      dec->connection_speed);
+  GST_DEBUG_OBJECT (urisrc, "redirect message: %" GST_PTR_FORMAT, msg);
+  GST_DEBUG_OBJECT (urisrc, "connection speed: %" G_GUINT64_FORMAT,
+      urisrc->connection_speed);
 
   structure = gst_message_get_structure (msg);
-  if (dec->connection_speed == 0 || structure == NULL)
+  if (urisrc->connection_speed == 0 || structure == NULL)
     return msg;
 
   locations_list = gst_structure_get_value (structure, "locations");
@@ -2067,13 +2296,13 @@ handle_redirect_message (GstURISourceBin * dec, GstMessage * msg)
     location_val = gst_value_list_get_value (locations_list, i);
     s = (const GstStructure *) g_value_get_boxed (location_val);
     if (!gst_structure_get_int (s, "minimum-bitrate", &bitrate) || bitrate <= 0) {
-      GST_DEBUG_OBJECT (dec, "no bitrate: %" GST_PTR_FORMAT, s);
+      GST_DEBUG_OBJECT (urisrc, "no bitrate: %" GST_PTR_FORMAT, s);
       l_neutral = g_list_append (l_neutral, (gpointer) s);
-    } else if (bitrate > dec->connection_speed) {
-      GST_DEBUG_OBJECT (dec, "bitrate too high: %" GST_PTR_FORMAT, s);
+    } else if (bitrate > urisrc->connection_speed) {
+      GST_DEBUG_OBJECT (urisrc, "bitrate too high: %" GST_PTR_FORMAT, s);
       l_bad = g_list_append (l_bad, (gpointer) s);
-    } else if (bitrate <= dec->connection_speed) {
-      GST_DEBUG_OBJECT (dec, "bitrate OK: %" GST_PTR_FORMAT, s);
+    } else if (bitrate <= urisrc->connection_speed) {
+      GST_DEBUG_OBJECT (urisrc, "bitrate OK: %" GST_PTR_FORMAT, s);
       l_good = g_list_append (l_good, (gpointer) s);
     }
   }
@@ -2091,7 +2320,7 @@ handle_redirect_message (GstURISourceBin * dec, GstMessage * msg)
   new_msg = gst_message_new_element (msg->src, new_structure);
   gst_message_unref (msg);
 
-  GST_DEBUG_OBJECT (dec, "new redirect message: %" GST_PTR_FORMAT, new_msg);
+  GST_DEBUG_OBJECT (urisrc, "new redirect message: %" GST_PTR_FORMAT, new_msg);
   return new_msg;
 }
 
@@ -2307,7 +2536,7 @@ typedef void (*QueryDoneFunction) (GstURISourceBin * urisrc, QueryFold * fold);
 /* for duration/position we collect all durations/positions and take
  * the MAX of all valid results */
 static void
-decoder_query_init (GstURISourceBin * dec, QueryFold * fold)
+uri_source_query_init (GstURISourceBin * urisrc, QueryFold * fold)
 {
   fold->min = 0;
   fold->max = -1;
@@ -2316,7 +2545,7 @@ decoder_query_init (GstURISourceBin * dec, QueryFold * fold)
 }
 
 static gboolean
-decoder_query_duration_fold (const GValue * item, GValue * ret,
+uri_source_query_duration_fold (const GValue * item, GValue * ret,
     QueryFold * fold)
 {
   GstPad *pad = g_value_get_object (item);
@@ -2337,7 +2566,7 @@ decoder_query_duration_fold (const GValue * item, GValue * ret,
 }
 
 static void
-decoder_query_duration_done (GstURISourceBin * dec, QueryFold * fold)
+uri_source_query_duration_done (GstURISourceBin * urisrc, QueryFold * fold)
 {
   GstFormat format;
 
@@ -2349,7 +2578,7 @@ decoder_query_duration_done (GstURISourceBin * dec, QueryFold * fold)
 }
 
 static gboolean
-decoder_query_position_fold (const GValue * item, GValue * ret,
+uri_source_query_position_fold (const GValue * item, GValue * ret,
     QueryFold * fold)
 {
   GstPad *pad = g_value_get_object (item);
@@ -2371,7 +2600,7 @@ decoder_query_position_fold (const GValue * item, GValue * ret,
 }
 
 static void
-decoder_query_position_done (GstURISourceBin * dec, QueryFold * fold)
+uri_source_query_position_done (GstURISourceBin * urisrc, QueryFold * fold)
 {
   GstFormat format;
 
@@ -2379,11 +2608,12 @@ decoder_query_position_done (GstURISourceBin * dec, QueryFold * fold)
   /* store max in query result */
   gst_query_set_position (fold->query, format, fold->max);
 
-  GST_DEBUG_OBJECT (dec, "max position %" G_GINT64_FORMAT, fold->max);
+  GST_DEBUG_OBJECT (urisrc, "max position %" G_GINT64_FORMAT, fold->max);
 }
 
 static gboolean
-decoder_query_latency_fold (const GValue * item, GValue * ret, QueryFold * fold)
+uri_source_query_latency_fold (const GValue * item, GValue * ret,
+    QueryFold * fold)
 {
   GstPad *pad = g_value_get_object (item);
 
@@ -2418,12 +2648,12 @@ decoder_query_latency_fold (const GValue * item, GValue * ret, QueryFold * fold)
 }
 
 static void
-decoder_query_latency_done (GstURISourceBin * dec, QueryFold * fold)
+uri_source_query_latency_done (GstURISourceBin * urisrc, QueryFold * fold)
 {
   /* store max in query result */
   gst_query_set_latency (fold->query, fold->live, fold->min, fold->max);
 
-  GST_DEBUG_OBJECT (dec,
+  GST_DEBUG_OBJECT (urisrc,
       "latency min %" GST_TIME_FORMAT ", max %" GST_TIME_FORMAT
       ", live %d", GST_TIME_ARGS (fold->min), GST_TIME_ARGS (fold->max),
       fold->live);
@@ -2431,7 +2661,8 @@ decoder_query_latency_done (GstURISourceBin * dec, QueryFold * fold)
 
 /* we are seekable if all srcpads are seekable */
 static gboolean
-decoder_query_seeking_fold (const GValue * item, GValue * ret, QueryFold * fold)
+uri_source_query_seeking_fold (const GValue * item, GValue * ret,
+    QueryFold * fold)
 {
   GstPad *pad = g_value_get_object (item);
 
@@ -2451,19 +2682,20 @@ decoder_query_seeking_fold (const GValue * item, GValue * ret, QueryFold * fold)
 }
 
 static void
-decoder_query_seeking_done (GstURISourceBin * dec, QueryFold * fold)
+uri_source_query_seeking_done (GstURISourceBin * urisrc, QueryFold * fold)
 {
   GstFormat format;
 
   gst_query_parse_seeking (fold->query, &format, NULL, NULL, NULL);
   gst_query_set_seeking (fold->query, format, fold->seekable, 0, -1);
 
-  GST_DEBUG_OBJECT (dec, "seekable %d", fold->seekable);
+  GST_DEBUG_OBJECT (urisrc, "seekable %d", fold->seekable);
 }
 
 /* generic fold, return first valid result */
 static gboolean
-decoder_query_generic_fold (const GValue * item, GValue * ret, QueryFold * fold)
+uri_source_query_generic_fold (const GValue * item, GValue * ret,
+    QueryFold * fold)
 {
   GstPad *pad = g_value_get_object (item);
   gboolean res;
@@ -2483,7 +2715,7 @@ decoder_query_generic_fold (const GValue * item, GValue * ret, QueryFold * fold)
 static gboolean
 gst_uri_source_bin_query (GstElement * element, GstQuery * query)
 {
-  GstURISourceBin *decoder;
+  GstURISourceBin *urisrc;
   gboolean res = FALSE;
   GstIterator *iter;
   GstIteratorFoldFunction fold_func;
@@ -2493,36 +2725,36 @@ gst_uri_source_bin_query (GstElement * element, GstQuery * query)
   GValue ret = { 0 };
   gboolean default_ret = FALSE;
 
-  decoder = GST_URI_SOURCE_BIN (element);
+  urisrc = GST_URI_SOURCE_BIN (element);
 
   switch (GST_QUERY_TYPE (query)) {
     case GST_QUERY_DURATION:
       /* iterate and collect durations */
-      fold_func = (GstIteratorFoldFunction) decoder_query_duration_fold;
-      fold_init = decoder_query_init;
-      fold_done = decoder_query_duration_done;
+      fold_func = (GstIteratorFoldFunction) uri_source_query_duration_fold;
+      fold_init = uri_source_query_init;
+      fold_done = uri_source_query_duration_done;
       break;
     case GST_QUERY_POSITION:
       /* iterate and collect durations */
-      fold_func = (GstIteratorFoldFunction) decoder_query_position_fold;
-      fold_init = decoder_query_init;
-      fold_done = decoder_query_position_done;
+      fold_func = (GstIteratorFoldFunction) uri_source_query_position_fold;
+      fold_init = uri_source_query_init;
+      fold_done = uri_source_query_position_done;
       break;
     case GST_QUERY_LATENCY:
       /* iterate and collect durations */
-      fold_func = (GstIteratorFoldFunction) decoder_query_latency_fold;
-      fold_init = decoder_query_init;
-      fold_done = decoder_query_latency_done;
+      fold_func = (GstIteratorFoldFunction) uri_source_query_latency_fold;
+      fold_init = uri_source_query_init;
+      fold_done = uri_source_query_latency_done;
       default_ret = TRUE;
       break;
     case GST_QUERY_SEEKING:
       /* iterate and collect durations */
-      fold_func = (GstIteratorFoldFunction) decoder_query_seeking_fold;
-      fold_init = decoder_query_init;
-      fold_done = decoder_query_seeking_done;
+      fold_func = (GstIteratorFoldFunction) uri_source_query_seeking_fold;
+      fold_init = uri_source_query_init;
+      fold_done = uri_source_query_seeking_done;
       break;
     default:
-      fold_func = (GstIteratorFoldFunction) decoder_query_generic_fold;
+      fold_func = (GstIteratorFoldFunction) uri_source_query_generic_fold;
       break;
   }
 
@@ -2536,7 +2768,7 @@ gst_uri_source_bin_query (GstElement * element, GstQuery * query)
       query, GST_QUERY_TYPE (query));
 
   if (fold_init)
-    fold_init (decoder, &fold_data);
+    fold_init (urisrc, &fold_data);
 
   while (TRUE) {
     GstIteratorResult ires;
@@ -2547,14 +2779,14 @@ gst_uri_source_bin_query (GstElement * element, GstQuery * query)
       case GST_ITERATOR_RESYNC:
         gst_iterator_resync (iter);
         if (fold_init)
-          fold_init (decoder, &fold_data);
+          fold_init (urisrc, &fold_data);
         g_value_set_boolean (&ret, default_ret);
         break;
       case GST_ITERATOR_OK:
       case GST_ITERATOR_DONE:
         res = g_value_get_boolean (&ret);
         if (fold_done != NULL && res)
-          fold_done (decoder, &fold_data);
+          fold_done (urisrc, &fold_data);
         goto done;
       default:
         res = FALSE;
