@@ -758,7 +758,7 @@ static void
 subrip_fix_up_markup (gchar ** p_txt, gconstpointer allowed_tags_ptr)
 {
   gchar *cur, *next_tag;
-  gchar *open_tags[32];
+  GPtrArray *open_tags = NULL;
   guint num_open_tags = 0;
   const gchar *iter_tag;
   guint offset = 0;
@@ -771,6 +771,7 @@ subrip_fix_up_markup (gchar ** p_txt, gconstpointer allowed_tags_ptr)
 
   g_assert (*p_txt != NULL);
 
+  open_tags = g_ptr_array_new_with_free_func (g_free);
   cur = *p_txt;
   while (*cur != '\0') {
     next_tag = strchr (cur, '<');
@@ -800,7 +801,7 @@ subrip_fix_up_markup (gchar ** p_txt, gconstpointer allowed_tags_ptr)
       index++;
       if (offset) {
         /* OK we found a tag, let's keep track of it */
-        open_tags[num_open_tags] = g_strdup (iter_tag);
+        g_ptr_array_add (open_tags, g_ascii_strdown (iter_tag, -1));
         ++num_open_tags;
         break;
       }
@@ -815,15 +816,17 @@ subrip_fix_up_markup (gchar ** p_txt, gconstpointer allowed_tags_ptr)
     if (*next_tag == '<' && *(next_tag + 1) == '/') {
       end_tag = strchr (cur, '>');
       if (end_tag) {
+        const gchar *last = NULL;
+        if (num_open_tags > 0)
+          last = g_ptr_array_index (open_tags, num_open_tags - 1);
         if (num_open_tags == 0
-            || g_ascii_strncasecmp (end_tag - 1, open_tags[num_open_tags - 1],
-                strlen (open_tags[num_open_tags - 1]))) {
-          GST_LOG ("broken input, closing tag '%s' is not open", next_tag);
+            || g_ascii_strncasecmp (end_tag - 1, last, strlen (last))) {
+          GST_LOG ("broken input, closing tag '%s' is not open", end_tag - 1);
           memmove (next_tag, end_tag + 1, strlen (end_tag) + 1);
           next_tag -= strlen (end_tag);
         } else {
           --num_open_tags;
-          g_free (open_tags[num_open_tags]);
+          g_ptr_array_remove_index (open_tags, num_open_tags);
         }
       }
     }
@@ -836,17 +839,18 @@ subrip_fix_up_markup (gchar ** p_txt, gconstpointer allowed_tags_ptr)
 
     s = g_string_new (*p_txt);
     while (num_open_tags > 0) {
-      GST_LOG ("adding missing closing tag '%s'", open_tags[num_open_tags - 1]);
+      GST_LOG ("adding missing closing tag '%s'",
+          (char *) g_ptr_array_index (open_tags, num_open_tags - 1));
       g_string_append_c (s, '<');
       g_string_append_c (s, '/');
-      g_string_append (s, open_tags[num_open_tags - 1]);
+      g_string_append (s, g_ptr_array_index (open_tags, num_open_tags - 1));
       g_string_append_c (s, '>');
-      g_free (open_tags[num_open_tags - 1]);
       --num_open_tags;
     }
     g_free (*p_txt);
     *p_txt = g_string_free (s, FALSE);
   }
+  g_ptr_array_free (open_tags, TRUE);
 }
 
 static gboolean
@@ -880,7 +884,13 @@ parse_subrip_time (const gchar * ts_string, GstClockTime * t)
 
   /* make sure we have exactly three digits after he comma */
   p = strchr (s, ',');
-  g_assert (p != NULL);
+  if (p == NULL) {
+    /* If there isn't a ',' the timestamp is broken */
+    /* https://gitlab.freedesktop.org/gstreamer/gst-plugins-base/issues/532#note_100179 */
+    GST_WARNING ("failed to parse subrip timestamp string '%s'", s);
+    return FALSE;
+  }
+
   ++p;
   len = strlen (p);
   if (len > 3) {
@@ -1851,7 +1861,8 @@ gst_sub_parse_sink_event (GstPad * pad, GstObject * parent, GstEvent * event)
       if (self->parser_type == GST_SUB_PARSE_FORMAT_SUBRIP ||
           self->parser_type == GST_SUB_PARSE_FORMAT_TMPLAYER ||
           self->parser_type == GST_SUB_PARSE_FORMAT_MPL2 ||
-          self->parser_type == GST_SUB_PARSE_FORMAT_QTTEXT) {
+          self->parser_type == GST_SUB_PARSE_FORMAT_QTTEXT ||
+          self->parser_type == GST_SUB_PARSE_FORMAT_VTT) {
         gchar term_chars[] = { '\n', '\n', '\0' };
         GstBuffer *buf = gst_buffer_new_and_alloc (2 + 1);
 
