@@ -60,6 +60,7 @@ cleanup_appsrc (GstElement * appsrc)
 {
   GST_DEBUG ("cleanup_appsrc");
 
+  gst_check_drop_buffers ();
   gst_check_teardown_sink_pad (appsrc);
   gst_check_teardown_element (appsrc);
 }
@@ -147,14 +148,14 @@ on_new_sample_from_source (GstAppSink * elt, gpointer user_data)
 }
 
 /*
- * appsink => appsrc pipelines executed 100 times: 
+ * appsink => appsrc pipelines executed 100 times:
  * - appsink pipeline has sync=false
  * - appsrc pipeline has sync=true
  * - appsrc has block=true
  * after 1 second an error message is posted on appsink pipeline bus
  * when the error is received the appsrc pipeline is set to NULL
  * and then the appsink pipeline is
- * set to NULL too, this must not deadlock 
+ * set to NULL too, this must not deadlock
  */
 
 GST_START_TEST (test_appsrc_block_deadlock)
@@ -418,6 +419,193 @@ GST_START_TEST (test_appsrc_caps_in_push_modes)
 
 GST_END_TEST;
 
+/* This test simulates a pipeline blocked pushing caps using a blocking pad
+ * probe. This state is seen if the application push buffers and later change
+ * the caps on one stream before the other stream have prerolled. In this
+ * state, GStreamer 1.12 and previous would deadlock inside GstBaseSrc as
+ * it was holding the live lock while calling create(). AppSrc serialize the
+ * caps event into it's queue and then push it downstream when create() is
+ * called. */
+
+static GstPadProbeReturn
+caps_event_probe_cb (GstPad * pad, GstPadProbeInfo * info, gpointer user_data)
+{
+  GMainLoop *loop = user_data;
+
+  if (GST_EVENT_TYPE (info->data) == GST_EVENT_CAPS) {
+    g_main_loop_quit (loop);
+    return GST_PAD_PROBE_OK;
+  }
+
+  return GST_PAD_PROBE_PASS;
+}
+
+GST_START_TEST (test_appsrc_blocked_on_caps)
+{
+  GstElement *pipeline = NULL, *app = NULL;
+  GstPad *pad = NULL;
+  GstCaps *caps = NULL;
+  GError *error = NULL;
+  GMainLoop *loop;
+
+  loop = g_main_loop_new (NULL, FALSE);
+
+  pipeline = gst_parse_launch ("appsrc is-live=1 name=app ! fakesink", &error);
+  g_assert_no_error (error);
+
+  app = gst_bin_get_by_name (GST_BIN (pipeline), "app");
+  pad = gst_element_get_static_pad (app, "src");
+
+  gst_pad_add_probe (pad,
+      GST_PAD_PROBE_TYPE_BLOCK | GST_PAD_PROBE_TYPE_EVENT_DOWNSTREAM,
+      caps_event_probe_cb, loop, NULL);
+  gst_object_unref (app);
+  gst_object_unref (pad);
+
+  gst_element_set_state (pipeline, GST_STATE_PLAYING);
+
+  caps = gst_caps_from_string ("application/x-test");
+  gst_app_src_set_caps (GST_APP_SRC (app), caps);
+  gst_caps_unref (caps);
+
+  g_main_loop_run (loop);
+
+#if 0
+  /* This would work around the issue by deblocking the source on older
+   * version of GStreamer */
+  gst_element_send_event (app, gst_event_new_flush_start ());
+#endif
+
+  /* As appsrc change the caps GstBaseSrc::create() virtual function, the live
+   * lock use to remains held and prevented the state change from happening. */
+  gst_element_set_state (pipeline, GST_STATE_NULL);
+  gst_object_unref (pipeline);
+  g_main_loop_unref (loop);
+}
+
+GST_END_TEST;
+
+static guint expect_offset;
+static gboolean chainlist_called;
+static gboolean done;
+
+static gboolean
+event_func (GstPad * pad, GstObject * parent, GstEvent * event)
+{
+  GST_LOG ("event %" GST_PTR_FORMAT, event);
+  if (GST_EVENT_TYPE (event) == GST_EVENT_EOS) {
+    g_mutex_lock (&check_mutex);
+    done = TRUE;
+    g_cond_signal (&check_cond);
+    g_mutex_unlock (&check_mutex);
+  }
+  gst_event_unref (event);
+  return TRUE;
+}
+
+static GstFlowReturn
+chain_____func (GstPad * pad, GstObject * parent, GstBuffer * buf)
+{
+  GST_LOG ("  buffer # %3u", (guint) GST_BUFFER_OFFSET (buf));
+
+  fail_unless_equals_int (GST_BUFFER_OFFSET (buf), expect_offset);
+  ++expect_offset;
+  gst_buffer_unref (buf);
+
+  return GST_FLOW_OK;
+}
+
+static GstFlowReturn
+chainlist_func (GstPad * pad, GstObject * parent, GstBufferList * list)
+{
+  guint i, len;
+
+  len = gst_buffer_list_length (list);
+
+  GST_DEBUG ("buffer list with %u buffers", len);
+  for (i = 0; i < len; ++i) {
+    GstBuffer *buf = gst_buffer_list_get (list, i);
+    GST_LOG ("  buffer # %3u", (guint) GST_BUFFER_OFFSET (buf));
+
+    fail_unless_equals_int (GST_BUFFER_OFFSET (buf), expect_offset);
+    ++expect_offset;
+  }
+  chainlist_called = TRUE;
+  gst_buffer_list_unref (list);
+  return GST_FLOW_OK;
+}
+
+GST_START_TEST (test_appsrc_push_buffer_list)
+{
+  GstElement *src;
+  guint i;
+
+  src = gst_element_factory_make ("appsrc", "appsrc");
+
+  mysinkpad = gst_check_setup_sink_pad (src, &sinktemplate);
+  gst_pad_set_chain_function (mysinkpad, chain_____func);
+  gst_pad_set_chain_list_function (mysinkpad, chainlist_func);
+  gst_pad_set_event_function (mysinkpad, event_func);
+  gst_pad_set_active (mysinkpad, TRUE);
+
+  expect_offset = 0;
+  chainlist_called = FALSE;
+  done = FALSE;
+
+  gst_element_set_state (src, GST_STATE_PLAYING);
+
+#define NUM_BUFFERS 100
+
+  for (i = 0; i < NUM_BUFFERS; ++i) {
+    GstFlowReturn flow;
+    GstBuffer *buf;
+
+    buf = gst_buffer_new ();
+    GST_BUFFER_OFFSET (buf) = i;
+
+    if (i == 0 || g_random_boolean ()) {
+      GstBufferList *buflist = gst_buffer_list_new ();
+
+      gst_buffer_list_add (buflist, buf);
+
+      buf = gst_buffer_new ();
+      GST_BUFFER_OFFSET (buf) = ++i;
+      gst_buffer_list_add (buflist, buf);
+      if (g_random_boolean ()) {
+        flow = gst_app_src_push_buffer_list (GST_APP_SRC (src), buflist);
+      } else {
+        g_signal_emit_by_name (src, "push-buffer-list", buflist, &flow);
+        gst_buffer_list_unref (buflist);
+      }
+    } else {
+      flow = gst_app_src_push_buffer (GST_APP_SRC (src), buf);
+    }
+    fail_unless_equals_int (flow, GST_FLOW_OK);
+  }
+
+  gst_app_src_end_of_stream (GST_APP_SRC (src));
+
+  g_mutex_lock (&check_mutex);
+  while (!done)
+    g_cond_wait (&check_cond, &check_mutex);
+  g_mutex_unlock (&check_mutex);
+
+  gst_element_set_state (src, GST_STATE_NULL);
+
+  /* make sure the buffer list was pushed out as list! */
+  fail_unless (chainlist_called);
+
+  /* can be NUM_BUFFERS or NUM_BUFFERS + 1 depending on whether last item
+   * was buffer list or not */
+  fail_unless (expect_offset >= NUM_BUFFERS);
+
+  gst_check_teardown_sink_pad (src);
+
+  gst_object_unref (src);
+}
+
+GST_END_TEST;
+
 static Suite *
 appsrc_suite (void)
 {
@@ -427,6 +615,8 @@ appsrc_suite (void)
   tcase_add_test (tc_chain, test_appsrc_non_null_caps);
   tcase_add_test (tc_chain, test_appsrc_set_caps_twice);
   tcase_add_test (tc_chain, test_appsrc_caps_in_push_modes);
+  tcase_add_test (tc_chain, test_appsrc_blocked_on_caps);
+  tcase_add_test (tc_chain, test_appsrc_push_buffer_list);
 
   if (RUNNING_ON_VALGRIND)
     tcase_add_loop_test (tc_chain, test_appsrc_block_deadlock, 0, 5);
