@@ -14,16 +14,14 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-/* #include <fcntl.h> */
-#include <unistd.h>
 #include <glib.h>
 #include <glib/gstdio.h>
 #include <gst/check/gstcheck.h>
@@ -31,15 +29,31 @@
 #include <gst/pbutils/encoding-profile.h>
 #include <gst/pbutils/encoding-target.h>
 
+#ifdef G_OS_UNIX
+#include <unistd.h>             /* For R_OK etc. */
+#endif
+
+static inline gboolean
+gst_caps_is_equal_unref (GstCaps * caps1, GstCaps * caps2)
+{
+  gboolean ret;
+
+  ret = gst_caps_is_equal (caps1, caps2);
+  gst_caps_unref (caps1);
+
+  return ret;
+}
+
 #define CHECK_PROFILE(profile, name, description, format, preset, presence, restriction) \
   {									\
   fail_if(profile == NULL);						\
   fail_unless_equals_string (gst_encoding_profile_get_name (profile), name); \
   fail_unless_equals_string (gst_encoding_profile_get_description (profile), description); \
-  fail_unless (gst_caps_is_equal (gst_encoding_profile_get_format (profile), format)); \
+  fail_unless (gst_caps_is_equal_unref (gst_encoding_profile_get_format (profile), format)); \
   fail_unless_equals_string (gst_encoding_profile_get_preset (profile), preset); \
   fail_unless_equals_int (gst_encoding_profile_get_presence (profile), presence); \
-  fail_unless (gst_caps_is_equal (gst_encoding_profile_get_restriction (profile), restriction)); \
+  if (restriction) \
+    fail_unless (gst_caps_is_equal_unref (gst_encoding_profile_get_restriction (profile), restriction)); \
   }
 
 GST_START_TEST (test_profile_creation)
@@ -50,9 +64,9 @@ GST_START_TEST (test_profile_creation)
   GstCaps *ogg, *vorbis, *theora;
   GstCaps *test1, *test2;
 
-  ogg = gst_caps_new_simple ("application/ogg", NULL);
-  vorbis = gst_caps_new_simple ("audio/x-vorbis", NULL);
-  theora = gst_caps_new_simple ("video/x-theora", NULL);
+  ogg = gst_caps_new_empty_simple ("application/ogg");
+  vorbis = gst_caps_new_empty_simple ("audio/x-vorbis");
+  theora = gst_caps_new_empty_simple ("video/x-theora");
 
   encprof = (GstEncodingProfile *) gst_encoding_container_profile_new ((gchar *)
       "ogg-theora-vorbis", "dumb-profile", ogg, (gchar *) "dumb-preset");
@@ -96,7 +110,7 @@ GST_START_TEST (test_profile_input_caps)
   GstCaps *vorbis;
   GstCaps *out, *restriction, *test1;
 
-  vorbis = gst_caps_new_simple ("audio/x-vorbis", NULL);
+  vorbis = gst_caps_new_empty_simple ("audio/x-vorbis");
 
   /* Simple case, no restriction */
   sprof = (GstEncodingProfile *)
@@ -110,7 +124,7 @@ GST_START_TEST (test_profile_input_caps)
   gst_encoding_profile_unref (sprof);
 
   /* One simple restriction */
-  restriction = gst_caps_from_string ("audio/x-raw-int,channels=2,rate=44100");
+  restriction = gst_caps_from_string ("audio/x-raw,channels=2,rate=44100");
   test1 = gst_caps_from_string ("audio/x-vorbis,channels=2,rate=44100");
   fail_if (restriction == NULL);
 
@@ -230,7 +244,7 @@ create_saveload_target (const gchar * targetname)
   gst_encoding_target_add_profile (target, profile);
 
   caps = gst_caps_from_string ("audio/x-pony-song,pretty=True");
-  caps2 = gst_caps_from_string ("audio/x-raw-int,channels=1,rate=44100");
+  caps2 = gst_caps_from_string ("audio/x-raw,channels=1,rate=44100");
   sprof =
       (GstEncodingProfile *) gst_encoding_audio_profile_new (caps, NULL, caps2,
       1);
@@ -241,8 +255,7 @@ create_saveload_target (const gchar * targetname)
 
   caps = gst_caps_from_string ("video/x-glitter,sparkling=True");
   caps2 =
-      gst_caps_from_string
-      ("video/x-raw-yuv,width=640,height=480,framerate=15/1");
+      gst_caps_from_string ("video/x-raw,width=640,height=480,framerate=15/1");
   sprof = (GstEncodingProfile *)
       gst_encoding_video_profile_new (caps, "seriously glittery", caps2, 0);
   gst_encoding_video_profile_set_variableframerate ((GstEncodingVideoProfile *)
@@ -291,7 +304,8 @@ GST_START_TEST (test_saving_profile)
   fail_unless (gst_encoding_target_save (orig, NULL));
 
   /* Check we can load it */
-  profile_file_name = g_build_filename (g_get_home_dir (), ".gstreamer-0.10",
+  profile_file_name =
+      g_build_filename (g_get_user_data_dir (), "gstreamer-1.0",
       "encoding-profiles", "herding", "myponytarget2.gep", NULL);
   GST_DEBUG ("Loading target from '%s'", profile_file_name);
   loaded = gst_encoding_target_load_from_file (profile_file_name, NULL);
@@ -364,7 +378,7 @@ test_individual_target (GstEncodingTarget * target)
 
   GST_DEBUG ("Checking the container profile has the audio/x-pony-song stream");
   tmpcaps = gst_caps_from_string ("audio/x-pony-song,pretty=True");
-  tmpcaps2 = gst_caps_from_string ("audio/x-raw-int,channels=1,rate=44100");
+  tmpcaps2 = gst_caps_from_string ("audio/x-raw,channels=1,rate=44100");
   sprof1 =
       (GstEncodingProfile *) gst_encoding_audio_profile_new (tmpcaps, NULL,
       tmpcaps2, 1);
@@ -377,8 +391,7 @@ test_individual_target (GstEncodingTarget * target)
   GST_DEBUG ("Checking the container profile has the video//x-glitter stream");
   tmpcaps = gst_caps_from_string ("video/x-glitter,sparkling=True");
   tmpcaps2 =
-      gst_caps_from_string
-      ("video/x-raw-yuv,width=640,height=480,framerate=15/1");
+      gst_caps_from_string ("video/x-raw,width=640,height=480,framerate=15/1");
   sprof2 = (GstEncodingProfile *)
       gst_encoding_video_profile_new (tmpcaps, "seriously glittery", tmpcaps2,
       0);
@@ -398,7 +411,7 @@ GST_START_TEST (test_loading_profile)
   GstEncodingProfile *profile;
   GstCaps *tmpcaps;
   GValue strvalue = { 0, };
-  GValue miniobjectvalue = { 0, };
+  GValue objectvalue = { 0, };
 
   gst_debug_set_threshold_for_name ("default", GST_LEVEL_NONE);
 
@@ -415,7 +428,8 @@ GST_START_TEST (test_loading_profile)
   gst_encoding_target_unref (target);
 
   /* Test loading using fully specified path */
-  profile_file_name = g_build_filename (g_get_home_dir (), ".gstreamer-0.10",
+  profile_file_name =
+      g_build_filename (g_get_user_data_dir (), "gstreamer-1.0",
       "encoding-profiles", "herding", "myponytarget.gep", NULL);
 
   GST_DEBUG ("Loading target from '%s'", profile_file_name);
@@ -446,13 +460,13 @@ GST_START_TEST (test_loading_profile)
 
   /* For my next trick, I will need the assistance of a GValue */
   g_value_init (&strvalue, G_TYPE_STRING);
-  g_value_init (&miniobjectvalue, GST_TYPE_ENCODING_PROFILE);
+  g_value_init (&objectvalue, GST_TYPE_ENCODING_PROFILE);
   g_value_set_static_string (&strvalue, "myponytarget/pony");
-  fail_unless (g_value_transform (&strvalue, &miniobjectvalue));
-  profile = (GstEncodingProfile *) gst_value_dup_mini_object (&miniobjectvalue);
+  fail_unless (g_value_transform (&strvalue, &objectvalue));
+  profile = (GstEncodingProfile *) g_value_dup_object (&objectvalue);
   fail_if (profile == NULL);
   g_value_unset (&strvalue);
-  g_value_unset (&miniobjectvalue);
+  g_value_unset (&objectvalue);
   tmpcaps = gst_caps_from_string ("animal/x-pony");
   CHECK_PROFILE (profile, "pony", "I don't want a description !", tmpcaps, NULL,
       0, 0);
@@ -493,7 +507,7 @@ GST_START_TEST (test_target_list)
   /* If tmp is NULL, it means we iterated the whole list without finding
    * our target */
   fail_if (tmp == NULL);
-  g_list_foreach (targets, (GFunc) gst_mini_object_unref, NULL);
+  g_list_foreach (targets, (GFunc) g_object_unref, NULL);
   g_list_free (targets);
 
   /* Try getting all available targets without a specified category */
@@ -507,7 +521,7 @@ GST_START_TEST (test_target_list)
   /* If tmp is NULL, it means we iterated the whole list without finding
    * our target */
   fail_if (tmp == NULL);
-  g_list_foreach (targets, (GFunc) gst_mini_object_unref, NULL);
+  g_list_foreach (targets, (GFunc) g_object_unref, NULL);
   g_list_free (targets);
 }
 
@@ -530,7 +544,7 @@ format=animal/x-pony\n\
 parent=pony\n\
 type=audio\n\
 format=audio/x-pony-song,pretty=True\n\
-restriction=audio/x-raw-int,channels=1,rate=44100\n\
+restriction=audio/x-raw,channels=1,rate=44100\n\
 presence=1\n\
 \n\
 [streamprofile-pony12]\n\
@@ -538,7 +552,7 @@ parent=pony\n\
 type=video\n\
 preset=seriously glittery\n\
 format=video/x-glitter,sparkling=True\n\
-restriction=video/x-raw-yuv,width=640,height=480,framerate=15/1\n\
+restriction=video/x-raw,width=640,height=480,framerate=15/1\n\
 presence=0\n\
 variableframerate=true\n\
 ";
@@ -548,11 +562,13 @@ remove_profile_file (void)
 {
   gchar *profile_file_name;
 
-  profile_file_name = g_build_filename (g_get_home_dir (), ".gstreamer-0.10",
+  profile_file_name =
+      g_build_filename (g_get_user_data_dir (), "gstreamer-1.0",
       "encoding-profiles", "herding", "myponytarget.gep", NULL);
   g_unlink (profile_file_name);
   g_free (profile_file_name);
-  profile_file_name = g_build_filename (g_get_home_dir (), ".gstreamer-0.10",
+  profile_file_name =
+      g_build_filename (g_get_user_data_dir (), "gstreamer-1.0",
       "encoding-profiles", "herding", "myponytarget2.gep", NULL);
   g_unlink (profile_file_name);
   g_free (profile_file_name);
@@ -566,12 +582,19 @@ create_profile_file (void)
   GError *error = NULL;
 
   profile_dir =
-      g_build_filename (g_get_home_dir (), ".gstreamer-0.10",
+      g_build_filename (g_get_user_data_dir (), "gstreamer-1.0",
       "encoding-profiles", "herding", NULL);
   profile_file_name =
-      g_build_filename (g_get_home_dir (), ".gstreamer-0.10",
+      g_build_filename (g_get_user_data_dir (), "gstreamer-1.0",
       "encoding-profiles", "herding", "myponytarget.gep", NULL);
+
+  /* on Windows it will ignore the mode anyway */
+#ifdef G_OS_WIN32
+  g_mkdir_with_parents (profile_dir, 0700);
+#else
   g_mkdir_with_parents (profile_dir, S_IRUSR | S_IWUSR | S_IXUSR);
+#endif
+
   if (!g_file_set_contents (profile_file_name, profile_string,
           strlen (profile_string), &error))
     GST_WARNING ("Couldn't write contents to file : %s", error->message);
@@ -591,6 +614,70 @@ test_teardown (void)
   remove_profile_file ();
 }
 
+GST_START_TEST (test_file_extension)
+{
+  GstEncodingContainerProfile *cprof;
+  GstCaps *ogg, *speex, *vorbis, *theora, *id3, *mp3;
+
+  /* 1 - ogg variants */
+  ogg = gst_caps_new_empty_simple ("application/ogg");
+  cprof = gst_encoding_container_profile_new ("myprofile", NULL, ogg, NULL);
+  gst_caps_unref (ogg);
+
+  fail_unless_equals_string (gst_encoding_profile_get_file_extension
+      (GST_ENCODING_PROFILE (cprof)), "ogg");
+
+  speex = gst_caps_new_empty_simple ("audio/x-speex");
+  gst_encoding_container_profile_add_profile (cprof,
+      (GstEncodingProfile *) gst_encoding_audio_profile_new (speex, NULL,
+          NULL, 1));
+  gst_caps_unref (speex);
+
+  fail_unless_equals_string (gst_encoding_profile_get_file_extension
+      (GST_ENCODING_PROFILE (cprof)), "spx");
+
+  vorbis = gst_caps_new_empty_simple ("audio/x-vorbis");
+  gst_encoding_container_profile_add_profile (cprof,
+      (GstEncodingProfile *) gst_encoding_audio_profile_new (vorbis, NULL,
+          NULL, 1));
+  gst_caps_unref (vorbis);
+
+  fail_unless_equals_string (gst_encoding_profile_get_file_extension
+      (GST_ENCODING_PROFILE (cprof)), "ogg");
+
+  theora = gst_caps_new_empty_simple ("video/x-theora");
+  gst_encoding_container_profile_add_profile (cprof,
+      (GstEncodingProfile *) gst_encoding_video_profile_new (theora, NULL,
+          NULL, 1));
+  gst_caps_unref (theora);
+
+  fail_unless_equals_string (gst_encoding_profile_get_file_extension
+      (GST_ENCODING_PROFILE (cprof)), "ogv");
+
+  gst_encoding_profile_unref (cprof);
+
+  /* 2 - tag container */
+  id3 = gst_caps_new_empty_simple ("application/x-id3");
+  cprof = gst_encoding_container_profile_new ("myprofile", NULL, id3, NULL);
+  gst_caps_unref (id3);
+
+  fail_unless (gst_encoding_profile_get_file_extension (GST_ENCODING_PROFILE
+          (cprof)) == NULL);
+
+  mp3 = gst_caps_new_simple ("audio/mpeg", "mpegversion", G_TYPE_INT, 1,
+      "layer", G_TYPE_INT, 3, NULL);
+  gst_encoding_container_profile_add_profile (cprof,
+      (GstEncodingProfile *) gst_encoding_audio_profile_new (mp3, NULL,
+          NULL, 1));
+  gst_caps_unref (mp3);
+
+  fail_unless_equals_string (gst_encoding_profile_get_file_extension
+      (GST_ENCODING_PROFILE (cprof)), "mp3");
+
+  gst_encoding_profile_unref (cprof);
+}
+
+GST_END_TEST;
 
 static Suite *
 profile_suite (void)
@@ -598,12 +685,18 @@ profile_suite (void)
   Suite *s = suite_create ("profile support library");
   TCase *tc_chain = tcase_create ("general");
   gboolean can_write;
-  gchar *gst_dir;
 
-  /* cehck if we can create profiles */
-  gst_dir = g_build_filename (g_get_home_dir (), ".gstreamer-0.10", NULL);
-  can_write = (g_access (gst_dir, R_OK | W_OK | X_OK) == 0);
-  g_free (gst_dir);
+  /* check if we can create profiles */
+#ifdef G_OS_UNIX
+  {
+    gchar *gst_dir =
+        g_build_filename (g_get_user_data_dir (), "gstreamer-1.0", NULL);
+    can_write = (g_access (gst_dir, R_OK | W_OK | X_OK) == 0);
+    g_free (gst_dir);
+  }
+#else
+  can_write = FALSE;            /* FIXME: fix can_write test on Windows */
+#endif
 
   suite_add_tcase (s, tc_chain);
 
@@ -611,6 +704,7 @@ profile_suite (void)
   tcase_add_test (tc_chain, test_profile_input_caps);
   tcase_add_test (tc_chain, test_target_naming);
   tcase_add_test (tc_chain, test_target_profile);
+  tcase_add_test (tc_chain, test_file_extension);
   if (can_write) {
     tcase_add_test (tc_chain, test_loading_profile);
     tcase_add_test (tc_chain, test_saving_profile);
