@@ -23,8 +23,8 @@
 
 #include <gst/gst.h>
 #include <gst/check/gstcheck.h>
-#include <gst/rtp/gstrtpbuffer.h>
-#include <gst/rtp/gstrtpbasedepayload.h>
+#include <gst/check/gstharness.h>
+#include <gst/rtp/rtp.h>
 
 #define DEFAULT_CLOCK_RATE (42)
 
@@ -123,7 +123,8 @@ gst_rtp_dummy_depay_process (GstRTPBaseDepayload * depayload, GstBuffer * buf)
     GstMemory *mem = gst_buffer_get_memory (buf, 0);
     gsize size, offset, maxsize;
     size = gst_memory_get_sizes (mem, &offset, &maxsize);
-    GST_LOG ("\tsize=%zd offset=%zd maxsize=%zd", size, offset, maxsize);
+    GST_LOG ("\tsize=%" G_GSIZE_FORMAT " offset=%" G_GSIZE_FORMAT " maxsize=%"
+        G_GSIZE_FORMAT, size, offset, maxsize);
     gst_memory_unref (mem);
   }
 
@@ -144,7 +145,8 @@ gst_rtp_dummy_depay_process (GstRTPBaseDepayload * depayload, GstBuffer * buf)
     GstMemory *mem = gst_buffer_get_memory (buf, 0);
     gsize size, offset, maxsize;
     size = gst_memory_get_sizes (mem, &offset, &maxsize);
-    GST_LOG ("\tsize=%zd offset=%zd maxsize=%zd", size, offset, maxsize);
+    GST_LOG ("\tsize=%" G_GSIZE_FORMAT " offset=%" G_GSIZE_FORMAT " maxsize=%"
+        G_GSIZE_FORMAT, size, offset, maxsize);
     gst_memory_unref (mem);
   }
 
@@ -317,22 +319,14 @@ validate_event (guint index, const gchar * name, const gchar * field, ...)
   va_end (var_args);
 }
 
-#define push_rtp_buffer(state, field, ...) \
-	push_rtp_buffer_full ((state), GST_FLOW_OK, (field), __VA_ARGS__)
-#define push_rtp_buffer_fails(state, error, field, ...) \
-        push_rtp_buffer_full ((state), (error), (field), __VA_ARGS__)
-
 static void
-push_rtp_buffer_full (State * state, GstFlowReturn expected,
-    const gchar * field, ...)
+rtp_buffer_set_valist (GstBuffer * buf, const gchar * field, va_list var_args,
+    gboolean * extra_ref_)
 {
-  GstBuffer *buf = gst_rtp_buffer_new_allocate (0, 0, 0);
   GstRTPBuffer rtp = { NULL };
   gboolean mapped = FALSE;
   gboolean extra_ref = FALSE;
-  va_list var_args;
 
-  va_start (var_args, field);
   while (field) {
     if (!g_strcmp0 (field, "pts")) {
       GstClockTime pts = va_arg (var_args, GstClockTime);
@@ -366,13 +360,18 @@ push_rtp_buffer_full (State * state, GstFlowReturn expected,
         gst_rtp_buffer_set_ssrc (&rtp, ssrc);
       } else if (!g_strcmp0 (field, "extra-ref")) {
         extra_ref = va_arg (var_args, gboolean);
+        if (extra_ref_)
+          *extra_ref_ = extra_ref;
+      } else if (!g_strcmp0 (field, "csrc")) {
+        guint idx = va_arg (var_args, guint);
+        guint csrc = va_arg (var_args, guint);
+        gst_rtp_buffer_set_csrc (&rtp, idx, csrc);
       } else {
         fail ("test cannot set unknown buffer field '%s'", field);
       }
     }
     field = va_arg (var_args, const gchar *);
   }
-  va_end (var_args);
 
   if (mapped) {
     gst_rtp_buffer_unmap (&rtp);
@@ -380,6 +379,34 @@ push_rtp_buffer_full (State * state, GstFlowReturn expected,
 
   if (extra_ref)
     gst_buffer_ref (buf);
+}
+
+static void
+rtp_buffer_set (GstBuffer * buf, const gchar * field, ...)
+{
+  va_list var_args;
+
+  va_start (var_args, field);
+  rtp_buffer_set_valist (buf, field, var_args, NULL);
+  va_end (var_args);
+}
+
+#define push_rtp_buffer(state, field, ...) \
+    push_rtp_buffer_full ((state), GST_FLOW_OK, (field), __VA_ARGS__)
+#define push_rtp_buffer_fails(state, error, field, ...) \
+    push_rtp_buffer_full ((state), (error), (field), __VA_ARGS__)
+
+static void
+push_rtp_buffer_full (State * state, GstFlowReturn expected,
+    const gchar * field, ...)
+{
+  GstBuffer *buf = gst_rtp_buffer_new_allocate (0, 0, 0);
+  va_list var_args;
+  gboolean extra_ref = FALSE;
+
+  va_start (var_args, field);
+  rtp_buffer_set_valist (buf, field, var_args, &extra_ref);
+  va_end (var_args);
 
   fail_unless_equals_int (gst_pad_push (state->srcpad, buf), expected);
 
@@ -388,7 +415,7 @@ push_rtp_buffer_full (State * state, GstFlowReturn expected,
 }
 
 #define push_buffer(state, field, ...) \
-	push_buffer_full ((state), GST_FLOW_OK, (field), __VA_ARGS__)
+    push_buffer_full ((state), GST_FLOW_OK, (field), __VA_ARGS__)
 
 static void
 push_buffer_full (State * state, GstFlowReturn expected,
@@ -509,7 +536,8 @@ set_state (State * state, GstState new_state)
 }
 
 static void
-packet_lost (State * state, GstClockTime timestamp, GstClockTime duration)
+packet_lost (State * state, GstClockTime timestamp, GstClockTime duration,
+    gboolean might_have_been_fec)
 {
   GstEvent *event;
   guint seqnum = 0x4243;
@@ -521,6 +549,7 @@ packet_lost (State * state, GstClockTime timestamp, GstClockTime duration)
           "seqnum", G_TYPE_UINT, seqnum,
           "timestamp", G_TYPE_UINT64, timestamp,
           "duration", G_TYPE_UINT64, duration,
+          "might-have-been-fec", G_TYPE_BOOLEAN, might_have_been_fec,
           "late", G_TYPE_BOOLEAN, late, "retry", G_TYPE_UINT, retries, NULL));
 
   fail_unless (gst_pad_push_event (state->srcpad, event));
@@ -869,7 +898,11 @@ GST_START_TEST (rtp_base_depayload_packet_lost_test)
       "pts", 0 * GST_SECOND,
       "rtptime", G_GUINT64_CONSTANT (0x1234), "seq", 0x4242, NULL);
 
-  packet_lost (state, 1 * GST_SECOND, GST_SECOND);
+  packet_lost (state, 1 * GST_SECOND, GST_SECOND, FALSE);
+
+  /* If a packet was lost but we don't know whether it was a FEC packet,
+   * the depayloader should not generate gap events */
+  packet_lost (state, 2 * GST_SECOND, GST_SECOND, TRUE);
 
   push_rtp_buffer (state,
       "pts", 2 * GST_SECOND,
@@ -901,6 +934,60 @@ GST_START_TEST (rtp_base_depayload_packet_lost_test)
 }
 
 GST_END_TEST
+/* If a lost event is received before the first buffer, the rtp base
+ * depayloader will not send a gap event downstream. Alternatively it should
+ * make sure that stream-start, caps and segment events are sent in correct
+ * order before the gap event so that packet loss concealment can take place
+ * downstream, but this is more complicated and without any real benefit since
+ * concealment before any data is received is not very useful. */
+GST_START_TEST (rtp_base_depayload_packet_lost_before_first_buffer_test)
+{
+  GstHarness *h;
+  GstEvent *event;
+  GstRtpDummyDepay *depay;
+  const GstEventType etype[] = {
+    GST_EVENT_STREAM_START, GST_EVENT_CAPS, GST_EVENT_SEGMENT
+  };
+  gint i;
+
+  depay = rtp_dummy_depay_new ();
+  h = gst_harness_new_with_element (GST_ELEMENT_CAST (depay), "sink", "src");
+  gst_harness_set_src_caps_str (h, "application/x-rtp");
+
+  /* Verify that depayloader has received setup events */
+  for (i = 0; i < 3; i++) {
+    event = gst_pad_get_sticky_event (h->srcpad, etype[i], 0);
+    fail_unless (event != NULL);
+    gst_event_unref (event);
+  }
+
+  /* Send loss event to depayloader */
+  gst_harness_push_event (h, gst_event_new_custom (GST_EVENT_CUSTOM_DOWNSTREAM,
+          gst_structure_new ("GstRTPPacketLost",
+              "seqnum", G_TYPE_UINT, (guint) 0,
+              "timestamp", G_TYPE_UINT64, (guint64) 0,
+              "duration", G_TYPE_UINT64, (guint64) 10 * GST_MSECOND, NULL)));
+
+  /* When a buffer is pushed, an updated (and more accurate) segment event
+   * should aslo be sent. */
+  gst_harness_push (h, gst_rtp_buffer_new_allocate (0, 0, 0));
+
+  /* Verify that setup events are sent before gap event */
+  for (i = 0; i < 3; i++) {
+    fail_unless (event = gst_harness_pull_event (h));
+    fail_unless_equals_int (GST_EVENT_TYPE (event), etype[i]);
+    gst_event_unref (event);
+  }
+  fail_unless_equals_int (gst_harness_events_in_queue (h), 0);
+
+  gst_buffer_unref (gst_harness_pull (h));
+  fail_unless_equals_int (gst_harness_buffers_in_queue (h), 0);
+
+  g_object_unref (depay);
+  gst_harness_teardown (h);
+}
+
+GST_END_TEST;
 /* rtp base depayloader should set DISCONT flag on buffer in case of a large
  * sequence number gap, and it's not set already by upstream. This tests a
  * certain code path where the buffer needs to be made writable to set the
@@ -1231,7 +1318,64 @@ GST_START_TEST (rtp_base_depayload_clock_base_test)
   destroy_depayloader (state);
 }
 
-GST_END_TEST static Suite *
+GST_END_TEST
+/* basedepayloader has a property source-info that will add
+ * GstRTPSourceMeta to the output buffer with RTP source information, such as
+ * SSRC and CSRCs. The is useful for letting downstream know about the origin
+ * of the stream. */
+GST_START_TEST (rtp_base_depayload_source_info_test)
+{
+  GstHarness *h;
+  GstRtpDummyDepay *depay;
+  GstBuffer *buffer;
+  GstRTPSourceMeta *meta;
+  guint seq = 0;
+
+  depay = rtp_dummy_depay_new ();
+  h = gst_harness_new_with_element (GST_ELEMENT_CAST (depay), "sink", "src");
+  gst_harness_set_src_caps_str (h, "application/x-rtp");
+
+  /* Property enabled should always add meta, also when there is only SSRC and
+   * no CSRC. */
+  g_object_set (depay, "source-info", TRUE, NULL);
+  buffer = gst_rtp_buffer_new_allocate (0, 0, 0);
+  rtp_buffer_set (buffer, "seq", seq++, "ssrc", 0x11, NULL);
+  buffer = gst_harness_push_and_pull (h, buffer);
+  fail_unless ((meta = gst_buffer_get_rtp_source_meta (buffer)));
+  fail_unless (meta->ssrc_valid);
+  fail_unless_equals_int (meta->ssrc, 0x11);
+  fail_unless_equals_int (meta->csrc_count, 0);
+  gst_buffer_unref (buffer);
+
+  /* Both SSRC and CSRC should be added to the meta */
+  buffer = gst_rtp_buffer_new_allocate (0, 0, 2);
+  rtp_buffer_set (buffer, "seq", seq++, "ssrc", 0x11, "csrc", 0, 0x22,
+      "csrc", 1, 0x33, NULL);
+  buffer = gst_harness_push_and_pull (h, buffer);
+  fail_unless ((meta = gst_buffer_get_rtp_source_meta (buffer)));
+  fail_unless (meta->ssrc_valid);
+  fail_unless_equals_int (meta->ssrc, 0x11);
+  fail_unless_equals_int (meta->csrc_count, 2);
+  fail_unless_equals_int (meta->csrc[0], 0x22);
+  fail_unless_equals_int (meta->csrc[1], 0x33);
+  gst_buffer_unref (buffer);
+
+  /* Property disabled should never add meta */
+  g_object_set (depay, "source-info", FALSE, NULL);
+  buffer = gst_rtp_buffer_new_allocate (0, 0, 0);
+  rtp_buffer_set (buffer, "seq", seq++, "ssrc", 0x11, NULL);
+  buffer = gst_harness_push_and_pull (h, buffer);
+  fail_if (gst_buffer_get_rtp_source_meta (buffer));
+  gst_buffer_unref (buffer);
+
+  g_object_unref (depay);
+  gst_harness_teardown (h);
+}
+
+GST_END_TEST;
+
+
+static Suite *
 rtp_basepayloading_suite (void)
 {
   Suite *s = suite_create ("rtp_base_depayloading_test");
@@ -1251,6 +1395,8 @@ rtp_basepayloading_suite (void)
   tcase_add_test (tc_chain, rtp_base_depayload_without_negotiation_test);
 
   tcase_add_test (tc_chain, rtp_base_depayload_packet_lost_test);
+  tcase_add_test (tc_chain,
+      rtp_base_depayload_packet_lost_before_first_buffer_test);
   tcase_add_test (tc_chain, rtp_base_depayload_seq_discont_test);
 
   tcase_add_test (tc_chain, rtp_base_depayload_repeated_caps_test);
@@ -1259,6 +1405,8 @@ rtp_basepayloading_suite (void)
   tcase_add_test (tc_chain, rtp_base_depayload_play_speed_test);
   tcase_add_test (tc_chain, rtp_base_depayload_clock_base_test);
 
+  tcase_add_test (tc_chain, rtp_base_depayload_source_info_test);
+
   return s;
 }
 
